/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Firefox.
 *
 * The Initial Developer of the Original Code is
 * Gavin Sharp.
 * Portions created by the Initial Developer are Copyright (C) 2006-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Gavin Sharp <gavin@gavinsharp.com> (Original author)
 *   Malte Kraus <mails@maltekraus.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
var AddToSearchBox = {
  init: function addToSearchBox__init() {
    this.replaceEngineFileCreation();
    var menu = document.getElementById("contentAreaContextMenu");
    menu.addEventListener("popupshowing", this.onPopupShowing, false);
  },

  replaceEngineFileCreation: function() {
    var xulRuntime = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime);
    if(xulRuntime == "WINNT") {
      var ss = Cc["@mozilla.org/browser/search-service;1"]
                 .getService(Ci.nsIBrowserSearchService);
      // search service global object:
      var topLevel = ss.defaultEngine.wrappedJSObject.__parent__;
      Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader)
        .loadSubScript("chrome://addtosearchbox/content/replaceEngineFileCreation.js",
                       topLevel);
    }
  },

  inputAddable: function() {
    if(!gContextMenu.onTextInput) return false;
    var form = gContextMenu.target.form, doc = form.ownerDocument;
    var formURI = makeURI(form.getAttribute("action"),
                          doc.characterSet, makeURI(doc.location.href));
    if(formURI.scheme != "javascript") {
      try {
        urlSecurityCheck(formURI, doc.nodePrincipal);
        return true;
      } catch(e) { }
    }
    return false;
  },
  onPopupShowing: function addToSearchBox__onPopupShowing(event) {
    if(event.target != event.currentTarget)
      return;
    gContextMenu.showItem("context-searchfield", AddToSearchBox.inputAddable());
  },

  _getStringPref: function(pref) {
    var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
    return prefs.getComplexValue(pref, Ci.nsISupportsString).data;
  },
  _getCharset: function addToSearchBox__getCharset(form, textToUri) {
    // pick the first charset mentioned in accept-charset known to Firefox:
    var charsets = form.acceptCharset.split(/[ \,]+/g);
    for(var i = 0; i < charsets.length; i++) {
      try {
        textToUri.ConvertAndEscape(charsets[i], ""); // test for valid charsets
        return charsets[i];
      } catch(e) {}
    }
    return form.ownerDocument.characterSet || this._getStringPref("intl.charset.default");
  },
  _collectAndAddParams: function(node, form, addedEngine, textToUri, charset) {
    var params = {};

    var addParam = function(name, value) {
      params[name] = value;
      // Firefox doesn't encode these on his own
      name = textToUri.ConvertAndEscape(charset, name);
      value = textToUri.ConvertAndEscape(charset, value);

      addedEngine.addParam(name, value, null);
    }

    var numButtons = 0; // we only send the first buttion, like when the user presses <enter>
    // Add the parameters from the form
    for (var i = 0; i < form.elements.length; ++i) {
      var el = form.elements[i];
      if(!el.name || el.disabled) // ignore these as per spec
        continue;

      var type = el.type.toLowerCase();
      if(el == node) {
        addedEngine.addParam(el.name, "{searchTerms}", null); // don't uri-encode this special value
        params[el.name] = "{searchTerms}";
      } else if(type == "text" || type == "hidden" || type == "textarea" ||
                type == "password" ||
                (el.checked && (type == "checkbox" || type == "radio")) ||
                ((type == "image" || type == "submit") && !numButtons++)) {
        addParam(el.name, el.value);
      } else if(type == "select-one" || type == "select-multiple") {
        // el instanceof el.ownerDocument.defaultView.HTMLSelectElement
        for(var j = 0; j < el.options.length; ++j) {
          if(el.options[j].selected)
            addParam(el.name, el.options[j].value);
        }
      }
    }
    return params;
  },

  addEngineToSearchBar: function BrowserSearch_add() {
    var node = document.popupNode;

    var addedEngineName = node.ownerDocument.defaultView.top.document.title;
    var form = node.form;
    var doc = node.ownerDocument, docURI = makeURI(doc.URL, doc.characterSet);
    var formURL = makeURI(form.getAttribute("action"), doc.characterSet, docURI).spec;
    var method = form.method.toUpperCase() == "POST" ? "POST" : "GET"; // search service is limited to these two
    if(formURL.indexOf("?") != -1 && method == "GET") // bug in search service?
      formURL = formURL + "&";
    var iconURL = gProxyFavIcon.getAttribute("src"); // current page's favicon
    if(!iconURL) {
      // favicon is only available when the page is fully loaded or has a <link rel="shortcut icon"> somewhere
      if(doc.location.host)
        iconURL = makeURI("/favicon.ico", doc.characterSet, docURI).spec;
      else
        iconURL = "chrome://addtosearchbox/skin/Search-bar.png";
    }

    window.openDialog("chrome://addtosearchbox/content/engineInfos.xul",
                      "_blank", "chrome,dependent,dialog,alwaysRaised,modal=0",
                      this.addEngineToSearchBarCallback,
                      [addedEngineName,iconURL,node,form,formURL,method]);
  },
  addEngineToSearchBarCallback: function(addedEngineName, iconURL, node, form,
                                         formURL, method, description, alias) {
    var ss = Cc["@mozilla.org/browser/search-service;1"]
               .getService(Ci.nsIBrowserSearchService);

    while(ss.getEngineByName(addedEngineName))
      addedEngineName += " "; // ugly, but working

    if(alias) { // only allow one alias
      var engWithAlias;
      while((engWithAlias = ss.getEngineByAlias(alias)))
        engWithAlias.alias = "";
    }

    try {
      ss.addEngineWithDetails(addedEngineName, iconURL, alias, description,
                              method, formURL);
    } catch(e) {
      alert(e.message);
      throw e;
    }
    // this should really be returned from addEngineWithDetails
    var addedEngine = ss.getEngineByName(addedEngineName);

    var textToUri = Cc["@mozilla.org/intl/texttosuburi;1"].getService(Ci.nsITextToSubURI);
    var charset = this._getCharset(form, textToUri);

    // the api lacks pretty basic stuff:
    var origEngineObj = addedEngine.wrappedJSObject;
    origEngineObj._searchForm = node.ownerDocument.location.href;
    origEngineObj._queryCharset = charset;

    var params = this._collectAndAddParams(node, form, addedEngine, textToUri, charset);

    origEngineObj._serializeToFile(); // work around bug 361061

    // Select the engine in the search bar
    ss.currentEngine = addedEngine;
  }
};
window.addEventListener("load", function() { AddToSearchBox.init(); }, false);