/* ***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is Search Engine Ordering.

The Initial Developer of the Original Code is
Malte Kraus.
Portions created by the Initial Developer are Copyright (C) 2006-2009
the Initial Developer. All Rights Reserved.

Contributor(s):
  Malte Kraus <mails@maltekraus.de> (Original author)

 Alternatively, the contents of this file may be used under the terms of
 either the GNU General Public License Version 2 or later (the "GPL"), or
 the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above. If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the GPL or the LGPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
***** END LICENSE BLOCK ***** */

const Cc = Components.classes, Ci = Components.interfaces, Cr = Components.results;

const CANVAS_DATAURL_PREFIX = "data:image/png;base64,";
const MAX_ICON_SIZE = 10000;

var callBack, args, addedEngineName, iconURL;
[callBack, args] = arguments;
[addedEngineName, iconURL,,,,] = args;

function pickIcon(e) {
  var prefs = Cc["@mozilla.org/preferences-service;1"]
                .getService(Ci.nsIPrefService)
                .getBranch("extensions.addtosearchbox.");
  var icon = document.getElementById("icon");

  var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
  fp.init(window, "", Ci.nsIFilePicker.modeOpen);
  fp.appendFilters(Ci.nsIFilePicker.filterAll | Ci.nsIFilePicker.filterImages);
  fp.filterIndex = 1; // images
  try {
    fp.displayDirectory = prefs.getComplexValue("lastdir", Ci.nsILocalFile);
    if(!fp.displayDirectory)
      throw new Error("jump to catch!");;
  } catch(e) {
    fp.displayDirectory = Cc["@mozilla.org/file/directory_service;1"]
                            .getService(Ci.nsIProperties)
                            .get("Home", Ci.nsIFile);
  }

  if(icon.file && icon.file instanceof Ci.nsILocalFile) {
    if(icon.file.parent.exists())
      fp.displayDirectory = icon.file.parent;
    if(icon.file.exists())
      fp.defaultString = icon.file.leafName;
  }

  var rv = fp.show();
  if(rv != Ci.nsIFilePicker.returnOK)
    return;
  var file = fp.file;
  prefs.setComplexValue("lastdir", Ci.nsILocalFile, file.parent);
  if(!file.exists() || !file.isReadable() || file.isDirectory())
    return;
  setIcon(fp.fileURL.spec, true);
  icon.file = file;
}


window.addEventListener("load", function onLoad(e) {
  document.getElementById("name").value = addedEngineName;
  var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);

  var icon = document.getElementById("icon");
  setIcon(iconURL, false, "chrome://addtosearchbox/skin/Search-bar.png");

  sizeToContent();
}, false);

function setIcon(url, verbose, fallback) {
  var image = new Image();
  image.onload = function() {
    var icon = document.getElementById("icon");
    var ctx = icon.getContext("2d");
    ctx.fillStyle = ctx.strokeStyle = "rgba(0,0,0,0)"; // transparent
    ctx.clearRect(0, 0, this.width, this.height);
    ctx.drawImage(image, 0, 0, 16, 16);
    var size = atob(icon.toDataURL().substr(CANVAS_DATAURL_PREFIX.length)).length;
    if(size > MAX_ICON_SIZE) {
      dump("Add to Search Bar:pickIcon icon too large (" + url + ")\n");
      if(verbose) {
        var sbs = Cc["@mozilla.org/intl/stringbundle;1"]
                    .getService(Ci.nsIStringBundleService);
        var stringBundle = sbs.createBundle("chrome://addtosearchbox/locale/engineInfos.properties");
        var title = stringBundle.GetStringFromName("fileSizeTitle");
        var sizes = [(MAX_ICON_SIZE / 1024).toFixed(2), (size / 1024).toFixed(2)];
        var text = stringBundle.formatStringFromName("fileSizeMessage", sizes, 2);
        var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Ci.nsIPromptService);
        prompts.alert(window, title, text);
      }
      if((fallback || icon.src) != url)
        setIcon(fallback || icon.src, false);
    } else {
      icon.src = icon.toDataURL();
      icon.setAttribute("tooltiptext", url);
    }
  };
  image.src = url;
}


function onDialogAccept(e) {
  args[0] = addedEngineName = document.getElementById("name").value;
  args[1] = document.getElementById("icon").src;
  args.push(addedEngineName); // description
  args.push(document.getElementById("alias").value);

  callBack.apply(opener.AddToSearchBox, args);
}
function onDialogCancel() { }