(function(){

  var cacheClass = Components.classes["@mozilla.org/network/cache-service;1"];
  var cacheService = cacheClass.getService(Components.interfaces.nsICacheService);

  var CacheToggle = {

    initialize: function() {
      this.log('CacheToggle: initializing...');
      
      this.diskCache          = Application.prefs.get("browser.cache.disk.enable", true);
      this.memCache           = Application.prefs.get("browser.cache.memory.enable", true);
      this.showTextLabel      = Application.prefs.get("extensions.cachetoggle.textLabel");
      this.debug              = Application.prefs.get("extensions.cachetoggle.debug");
      
      this.label              = document.getElementById("CacheToggle-Label");
      this.switchControl      = document.getElementById("CacheToggle-Switch");
      this.textLabelMenuItem  = document.getElementById("CacheToggle-ToggleTextLabelMenuItem");
      this.clearCacheMenuItem = document.getElementById("CacheToggle-ClearCacheMenuItem");
      
      this.memCache.value     = this.diskCache.value;

      this.label.style.display = this.showTextLabel.value ? "" : "none";
      this.switchControl.setOn(this.diskCache.value);
      this.textLabelMenuItem.setAttribute("checked", this.showTextLabel.value);

      this.boundToggle = function(event){ CacheToggle.toggle(event); };
      this.boundUpdate = function(event){ CacheToggle.updateSwitchControl(event); };

      this.switchControl.addEventListener("change", this.boundToggle, false);
      this.diskCache.events.addListener("change", this.boundUpdate, false);
      this.memCache.events.addListener("change", this.boundUpdate, false);
      this.textLabelMenuItem.addEventListener("command", function(event){ CacheToggle.toggleTextLabel(); }, false);
      this.clearCacheMenuItem.addEventListener("command", function(event){ CacheToggle.clearCache(); }, false);
    },

    toggle: function(event) {
      this.setCacheTo(event.target.isOn);
      this.log('CacheToggle: browser cache is enabled: ' + this.diskCache.value);
    },
    
    setCacheTo: function(value) {
      this.diskCache.events.removeListener("change", this.boundUpdate, false);
      this.memCache.events.removeListener("change", this.boundUpdate, false);
      this.memCache.value = this.diskCache.value = Boolean(value);
      this.diskCache.events.addListener("change", this.boundUpdate, false);
      this.memCache.events.addListener("change", this.boundUpdate, false);
    },

    toggleTextLabel: function() {
      this.showTextLabel.value = !this.showTextLabel.value;
      this.label.style.display = this.showTextLabel.value ? "" : "none";
    },

    updateSwitchControl: function(event) {
      var value = Application.prefs.get(event.data, true).value;
      this.switchControl.removeEventListener("change",this.boundToggle, false);
      this.switchControl.setOn(value);
      this.setCacheTo(value);
      this.switchControl.addEventListener("change",this.boundToggle, false);
    },
    
    clearCache: function() {
      cacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
      cacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
      this.log('CacheToggle: evicted cache entries');
    },
    
    destroy: function() {
      this.switchControl.removeEventListener("change",this.boundToggle, false);
      this.diskCache.events.removeListener("change", this.boundUpdate);
      this.log('CacheToggle: destroyed');
    },
    
    log: function(msg) {
      if (this.debug) {
        Application.console.log(msg);
      }
    }

  };

  window.addEventListener("load", function(){CacheToggle.initialize();}, false);
  window.addEventListener("unload", function(){CacheToggle.destroy();}, false);

})();