(function() {
	let ExtensionOptionsMenu = {
		startup: function() {
			ExtensionOptionsMenu.register();
			ExtensionOptionsMenu.init();
			window.addEventListener('load', ExtensionOptionsMenu.setup, false);
			window.addEventListener('customizationchange', ExtensionOptionsMenu.setup, false);
		},

		register: function() {
			let EOMPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			this._branch = EOMPref.getBranch("extensions.ExtensionOptionsMenu.");
			this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
			this._branch.addObserver("", this, false);
		},

		unregister: function() {
			if (!this._branch)
				return;
			this._branch.removeObserver("", this);
		},

		observe: function(aSubject, aTopic, aData) {
			if(aTopic = "nsPref:changed")
				ExtensionOptionsMenu.setup();
		},

		init: function() {
			Components.utils.import("resource://gre/modules/AddonManager.jsm");
			AddonManager.addAddonListener(ExtensionOptionsMenu);
		},

		onDisabled: function() {
			ExtensionOptionsMenu.setup();
		},

		onEnabled: function() {
			ExtensionOptionsMenu.setup();
		},

		onUninstalled: function() {
			ExtensionOptionsMenu.setup();
		},

		onInstalled: function() {
			ExtensionOptionsMenu.setup();
		},

		setup: function() {
			let EOMPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

			let EOToolsmenu = document.getElementById("extensionsoptionsmenu");
			EOToolsmenu.hidden = !EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.InToolsMenu");
			if (EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowIcons")) {
				EOToolsmenu.setAttribute("class", "menu-iconic");
				EOToolsmenu.setAttribute("style", "list-style-image: url(chrome://optionsmenu/skin/icon.png)");
			}
			else
				EOToolsmenu.setAttribute("class", "menu");

			ExtensionOptionsMenu.populate(EOToolsmenu.firstChild);

			let EOAppmenu = document.getElementById("extensionsoptionsAppmenu");
			if (EOAppmenu) {
				EOAppmenu.hidden = !EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.InAppMenu");
				document.getElementById("appmenu_addons").hidden = !EOAppmenu.hidden;

				ExtensionOptionsMenu.populate(EOAppmenu.firstChild);
			}

			let EOMToolbarButton = document.getElementById("extensionsoptionsmenutoolbar");
			if (EOMToolbarButton)
				ExtensionOptionsMenu.populate(EOMToolbarButton.firstChild);
		},

		populate: function(aParent) {
			AddonManager.getAllAddons(function(addon) {
				while (aParent.childNodes.length)
					aParent.removeChild(aParent.firstChild);

				let item, MenuIconURL, EOMShowIt = true, EOMBlackListArray = [];
				let EOMPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
				let EOshowVersion = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowVersion");
				let EOshowIcon = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowIcons");
				let EOMshowAll = EOMPref.getBoolPref("extensions.ExtensionOptionsMenu.ShowAll");
				let EOMopenApp = window['BrowserOpenAddonsMgr'] || window['openAddonsMgr'] || window['toEM'];
				let EOMBlackList = EOMPref.getCharPref("extensions.ExtensionOptionsMenu.BlackList");
				if (EOMBlackList)
					EOMBlackListArray = EOMBlackList.split(",");

				addon.forEach(function(EOMaddon) {
					if (EOMaddon.type == "extension" && EOMaddon.isActive && (EOMaddon.optionsURL || EOMshowAll)) {
						item = document.createElement("menuitem");
						item.setAttribute("label", EOshowVersion ? EOMaddon.name + "  " + EOMaddon.version : EOMaddon.name);
						item.setAttribute("tooltiptext", EOMaddon.description);
						EOMaddon.optionsURL ? item.addEventListener("command", function() { EOMaddon.optionsType == 2 ? EOMopenApp('addons://detail/' + encodeURIComponent(EOMaddon.id)) : openDialog(EOMaddon.optionsURL, '', 'chrome,titlebar,toolbar,resizable,scrollbars,centerscreen,dialog=no,modal=no') }, false) : item.setAttribute("disabled", true);
						if (EOshowIcon) {
			 				item.setAttribute("class", "menuitem-iconic");
							MenuIconURL = EOMaddon.iconURL || 'chrome://optionsmenu/skin/icon.png';
							item.setAttribute("style", "list-style-image: url(" + MenuIconURL + ")");
						}
						else
							item.setAttribute("class", "menuitem");

						if (EOMBlackListArray)
							for (let i = 0; i < EOMBlackListArray.length; i++ ) {
								if (EOMBlackListArray[i].toLowerCase().replace(/^ | $/g,"") == EOMaddon.name.toLowerCase()) {
									EOMShowIt = false;
									break;
								}
								else
									EOMShowIt = true;
							}
						if (EOMShowIt)
							aParent.appendChild(item);
					}
				});

				Components.classes["@mozilla.org/xul/xul-sort-service;1"].getService(Components.interfaces.nsIXULSortService).sort(aParent, "label", "ascending");
			});
		}
	};
	ExtensionOptionsMenu.startup();
})()