/************************************************************************/
/* Groundspeed                                                          */
/*   A Firefox Add-on for web application security testers              */
/*                                                                      */
/* author: Felipe Moreno (felipe@wobot.org)                             */
/* homepage: http://groundspeed.wobot.org                               */
/* v1.2.0 - January 2010                                                */
/*                                                                      */
/************************************************************************/
/* LICENSE INFORMATION                                                  */
/************************************************************************

Copyright (c) 2009-2010, Felipe Moreno
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
    * The names of its contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS 
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/************************************************************************/
/*                                                                      */
/*  BUGS:                                                               */
/*    - for pages with poor layout design (e.g. when the form tag is    */
/*      inside a table) the form fields will not be descendants of the  */
/*      FORM node in the DOM tree. In this case, groundspeed does not   */
/*      work because if we replace the form element by another element  */
/*      the new element will not be considered part of the original     */
/*      form. Note that the fact of highlighting the element needs      */
/*      replacing the node. Right now the solution is to detect,        */
/*      warn the user and do not add the element to groundspeed         */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/* DataModel object                                                     */
/************************************************************************/
/*  This object implements the nsITreeView interface to display         */
/*  the form elements in the GUI tree                                   */
/************************************************************************/

var gs_bundles = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
var gs_strings = gs_bundles.createBundle("chrome://gspeed/locale/groundspeed.properties");

function getLocaleString(name) {
	//return document.getElementById("gspeed_strings").getString(name);
	return gs_strings.GetStringFromName(name);
}

function GsDataModel() {};
GsDataModel.prototype = {
	GUIselected : null,
	forms : new Array(),
	highlighted : null,

	redoLog : [],

	savedFormValues : [],

	reset : function() {
		this.unmark();
		this.forms = new Array();
		this.treeBox.rowCountChanged(0, -this.visibleData.length);
		this.visibleData = new Array();

	},


	/* Code based on the XUL tutorial from XUL planet */
	/* To hold data for the TreeView */

	/* will not be necessary anymore, all info will come from forms array */
	childData : {
    		LoginForm: ["Username", "Password", "Remember Me"],
		SearchForm: ["Search"],
		GoToForm: ["Zip", "Country"]
	},

	/* will be an array of the visible gsElements */
	visibleData : new Array(),


	/* For the nsITreeView interface */
	treeBox: null,
	selection: null,

	get rowCount()                     { return this.visibleData.length; },
	setTree: function(treeBox)         { this.treeBox = treeBox; },
	getCellText: function(idx, column) { return this.visibleData[idx].createLabel(); }, 
	isContainer: function(idx)         { return this.visibleData[idx].isForm; }, 
	isContainerOpen: function(idx)     { return this.visibleData[idx].isOpen; }, 
	isContainerEmpty: function(idx)    { return false; },
	isSeparator: function(idx)         { return false; },
	isSorted: function()               { return false; },
	isEditable: function(idx, column)  { return false; },

	/* not implemented, from nsITreeView interface */
	getImageSrc: function(idx, column) {},
	getProgressMode : function(idx,column) {},
	getCellValue: function(idx, column) {},
	cycleHeader: function(col, elem) {},
	selectionChanged: function() {},
	cycleCell: function(idx, column) {},
	performAction: function(action) {},
	performActionOnCell: function(action, index, column) {},
	getRowProperties: function(idx, column, prop) {},
	getCellProperties: function(idx, column, prop) {},
	getColumnProperties: function(column, element, prop) {},


	getParentIndex: function(idx) {
		if (this.isContainer(idx)) return -1;
		for (var t = idx - 1; t >= 0 ; t--) {
			if (this.isContainer(t)) return t;
		}
	},

	getLevel: function(idx) {
		if (this.isContainer(idx)) return 0;
		return 1;
	},
	
	hasNextSibling: function(idx, after) {
		var thisLevel = this.getLevel(idx);
		for (var t = idx + 1; t < this.visibleData.length; t++) {
			var nextLevel = this.getLevel(t)
			if (nextLevel == thisLevel) return true;
			else if (nextLevel < thisLevel) return false;
		}
	},

	toggleOpenState: function(idx) { 
		var item = this.visibleData[idx];
		if (!item.isForm) return; 

		/* simply unmark the element when opening or closing a form */
		this.unmark();

		if (item.isOpen) { 
			item.isOpen = false;

			var thisLevel = this.getLevel(idx);
			var deletecount = 0;
			for (var t = idx + 1; t < this.visibleData.length; t++) {
				if (this.getLevel(t) > thisLevel) deletecount++;
				else break;
			}
			if (deletecount) {
				this.visibleData.splice(idx + 1, deletecount);
				this.treeBox.rowCountChanged(idx + 1, -deletecount);
				delta = -deletecount;
			}
		}
		else {
			item.isOpen = true; 
	
			var label = this.visibleData[idx].createLabel(); 

			var toinsert = this.visibleData[idx].children; 

			for (var i = 0; i < toinsert.length; i++) {
				this.visibleData.splice(idx + i + 1, 0, toinsert[i]);
			}
			this.treeBox.rowCountChanged(idx + 1, toinsert.length);
			delta = toinsert.length;
		}

	},
	
	repaint : function() {
		var boxobject = this.treeBox;
		boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
		this.treeBox.rowCountChanged(0, this.visibleData.length);

	},


	/* should this function add the form to visibleData? */
	addForm : function(aNode) {

		/* Create the form */
		var form = new GsElement(aNode);
		form.isForm = true;

		/* Grab document object from current selected tab */
		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
		var current_document = browserWindow.contentDocument;


		/* Go trough the form elements and add then to form object */

		/* Using the treewalker is a bad option because not all form fields are children of the form node
		   Better to use the elements property in the form tag!! */
		/*
		var iterator = current_document.createTreeWalker(aNode, NodeFilter.SHOW_ELEMENT, gsSelectFormInterestingTags, true);
		var j = 0;
			var input = iterator.nextNode();
		form.children = new Array();
		while (input) {*/

		var noWarn = true;
		var desc = "";

		for(var i = 0; i < aNode.elements.length; i++) {
			var input = aNode.elements[i];

			var inputElement = new GsElement(input);            		

			var end = false;
			desc = "";
			noWarn = true;
			//alert('new: '+ input.name + ' / ' + input.tagName + ', ' + noWarn);
			desc += "(" + input.name + ') ***BEGIN*** ' + input.tagName + ' - ';
			var dad = input.parentNode;
			while(!end) {
				//alert(dad.tagName + ', ' + noWarn);
				desc += dad.tagName + ' - ';
				if(dad.tagName.toUpperCase() == "FORM") {
					noWarn = false;
					end = true;
				}
				dad = dad.parentNode;
				if(dad.tagName.toUpperCase() == "BODY") end = true;
			}
			desc += "***END***";

			if(noWarn) {
				//alert("** Groundspeed WARNING! **\n\n" + inputElement.createLabel() + " in form " + form.getName() + " is not a descendant of the FORM element in the DOM Tree (this is caused by poor layout design, usually because the form definition is inside a table)\n\nAs this will cause problems when replacing or cloning the node (it will no longer be considered part of the original form) it will not be added to Groundspeed... You should use a proxy to test it.\n\nHere is some debug info: " + desc);

			} 
			else { /* only add the element to GroundSpeed if it's safe to mess with */
				//alert("Trace: " + desc);
				form.children.push(inputElement);
			}

		}	

		/* alert(form.children.length + " children added to the form"); */

		/* Add the form */
		this.forms.push(form);

		/* Add the form the visible part of the Tree */
		this.visibleData.push(form);

		return form;
	},

	mark : function(val, color) {

	//	alert("marking: " + val);
		if(this.highlighted == val) return;
		this.unmark();
		this.highlighted = val;
		this.visibleData[val].mark(color);

	},

	unmark : function() {
	//	alert("unmarking: " + this.highlighted);
		if(this.highlighted == null) return;
		this.visibleData[this.highlighted].unmark();
		this.highlighted = null;

	},

	getNode : function(val) {
		return this.visibleData[val];
	},

	performAction: function(action) {
	/* main function to add, remove or modify element and attributes
	   will offer redo and undo logs */

	/* add logging here... someday... */

		switch (action.type){
			case GSACTION_ADD_ELEMENT:
				//alert("add element " + action.params[0] + " with name " + action.params[1]);
				var tag = action.params.shift();
				var name = action.params.shift();
				action.targetElement.addElement(tag, name, action.params);
				break;

			case GSACTION_REMOVE_ELEMENT:
				//alert("Removing element");
				this.unmark();
				action.targetElement.removeElement();
				break;

			case GSACTION_REMOVE_ATTRIBUTE:
				//alert("remove attribute " + action.params[0]);
				var attribute = action.params[0];
				action.targetElement.removeAttribute(attribute);
				break;

			case GSACTION_ADD_ATTRIBUTE: /* right now, both are the same... */
			case GSACTION_CHANGE_ATTRIBUTE:
				//alert("change/add attribute " + action.params[0] + " to " + action.params[1]);
				var attribute = action.params[0];
				var newValue = action.params[1];
				action.targetElement.setAttribute(attribute, newValue);
				break;

			case GSACTION_CLONE_ELEMENT:
				this.unmark();
				//alert("creating by cloning and adding attributes");
				action.targetElement.cloneElement(action.params);
				break;
			case GSACTION_REPLACE_ELEMENT:
				this.unmark();
				var tag = action.params.shift();
				var cheatSheet = action.params.shift();
				action.targetElement.replaceElement(tag, cheatSheet, action.params);				
				break;
			default:
		}
	},

	saveFormValues : function(name, val) {
	
		var form = this.forms[val];
		var fieldArray = new Array();

		for(var i = 0; i < form.children.length; i++) {
			element = form.children[i];
			var savedField = new GsSavedField(element.node.tagName, element.node.getAttribute("name"), element.node.getAttribute("id"), element.node.value);
			//savedField.showAlert();
			fieldArray.push(savedField);
		}

		if(this.savedFormValues.length > 4) { this.savedFormValues.shift(); }
		this.savedFormValues.push([name, fieldArray]);
		//alert("saved sets of values: " + this.savedFormValues.length);
	}
}


/************************************************************************/
/* Saved field object                                                   */
/************************************************************************/
/*  Used by the save/load form values functionality                     */
/*                                                                      */
/************************************************************************/

function GsSavedField(t, n, i, v) {
	this.name = n;
	this.id = i;
	this.tagName = t;
	this.value = v;
}

GsSavedField.prototype = {
	name : null,
	id : null,
	tagName : null,
	value : null,

	showAlert : function() {
		alert("Saving [" + this.tagName + ", " + this.id + ", " + this.name + "] with value " + this.value);
	},

	equals : function(element) {

		var ename = element.getAttribute("name");
		var eid = element.getAttribute("id");
		var etag = element.tagName;

		//alert(ename + " agaisnt " + this.name);
		return ((ename == this.name) && (eid == this.id) && (etag == this.tagName));
	}

}


/************************************************************************/
/* Element object                                                       */
/************************************************************************/
/*  Used to represent a form or a form element in the data model        */
/*  object                                                              */
/************************************************************************/
function GsElement(aNode) {
	this.node = aNode;
	this.createLabel();
	this.children = new Array();
}

GsElement.prototype = {
	node : null,
	index : null,
	children : new Array(),
	marked : false,
	modified : false,
	label : "",
	isForm : false,
	isOpen : false,

	getType : function(){

		if(this.node.tagName.toUpperCase() == "INPUT") {
			var type;

			if(this.node.getAttributeNode("type"))
				type = this.node.getAttributeNode("type").value;
			else
				type = "TEXT";
			return type.toUpperCase();
		}
		else {
			return this.node.tagName;
		}

	},

	getName : function() {
		//var gs_strbundle = document.getElementById("gspeed_strings");
		var data = getLocaleString("unknownName");

		if(this.node.getAttributeNode("name")) {
			data = this.node.getAttributeNode("name").value;
		}
		else if(this.node.getAttributeNode("id")) {
			data = this.node.getAttributeNode("id").value;
		}
		return data;
	},

	/* Create label for listbox */
	createLabel : function() {
		//if(this.label != "") return this.label;
		this.label = this.getName() + " (" + this.getType() + ")";
		return this.label;
	},

	/* Functions to perform actions, called by DataModel.performAction */
	addElement : function(tag, name, params) {

		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
		var current_document = browserWindow.contentDocument;

		var new_element = current_document.createElement(tag);

		//var gs_strbundle = document.getElementById("gspeed_strings");
		var data = getLocaleString("new");

		var new_container = gsCreateMarkContainer(new_element, name + " ("+ data + " " + tag.toUpperCase() + ")" , "darkgreen", "");

		new_element.setAttribute("name", name);

		//alert(params.join(" / "));

		if(params.length > 0) {
			var attname = "";
			var attvalue = "";
			for(var i = 0; i < params.length; i+=2) {
				attname = params[i];
				if(params[i+1]) attvalue = params[i+1];
				else attvalue = "";
				new_element.setAttribute(attname, attvalue);
			}
		}
		this.node.appendChild(new_container);
	},

	removeElement : function() {
		this.node.parentNode.removeChild(this.node);
	},

	getElement : function() { },
	addAttribute : function() { },
	removeAttribute : function(attribute) { this.node.removeAttribute(attribute) },
	setAttribute : function(attribute, newValue) {
		if(attribute == "value") {
			/* if the attribute is the value, we should modify the value property... 
			the value attribute is the initial value read from the html and do not change */
			this.node.value = newValue;
		}
		else {
			this.node.setAttribute(attribute, newValue);
		}
	},
	getAttribute : function() { },

	replaceElement : function(tag, cheatSheet, params) {
		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
		var current_document = browserWindow.contentDocument;

		/* create a new element */
		var new_element = current_document.createElement(tag);

		/* add attributes */
		if(params.length > 0) {
			var attname = "";
			var attvalue = "";
			for(var i = 0; i < params.length; i+=2) {
				attname = params[i];
				if(params[i+1]) attvalue = params[i+1];
				else attvalue = "";
				new_element.setAttribute(attname, attvalue);
			}
		}

		/* create a new container */
		
		var new_container = gsCreateMarkContainer(new_element, this.getName() + " (" + getLocaleString("originally") + " " + this.getType() + ")" , "blue", cheatSheet); 
		
		/* replace element and set the new node */
		this.node.parentNode.replaceChild(new_container, this.node);
		this.node = new_element;
	},

	cloneElement : function(params) {
		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
		var current_document = browserWindow.contentDocument;

		var parent = this.parentNode;
		var clone = this.node.cloneNode(true); 

		if(params.length > 0) {
			var attname = "";
			var attvalue = "";
			for(var i = 0; i < params.length; i+=2) {
				attname = params[i];
				if(params[i+1]) attvalue = params[i+1];
				else attvalue = "";
				clone.setAttribute(attname, attvalue);
			}
		}

		var new_container = gsCreateMarkContainer(clone, this.getName() + " (" + getLocaleString("originally") + " " + this.getType() + ")" , "blue", "");

		/* replace current element by clone */
		this.node.parentNode.replaceChild(new_container, this.node);
		this.node = clone;
	},


	mark: function(color) {

		if(this.marked) this.unmark();

		this.marked = true;

		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
		var current_document = browserWindow.contentDocument;

		/* replace node by tempo node */
		var tempNode = current_document.createElement("tempNode");
		this.node.parentNode.replaceChild(tempNode, this.node);

		/* create container element */
		var table = current_document.createElement("table");
		table.setAttribute("style", "border: 1px solid " + color + "; font-family: sans-serif; font-size: small; padding: 0px; margin: 4px");

		var tr_header = current_document.createElement("tr");
		tr_header.setAttribute("style", "padding:0px;");
		var td_header = current_document.createElement("td");
		td_header.setAttribute("style", "background-color: " + color + "; color: white; font-weight: bold; padding: 2px 5px; margin: 0px");

		td_header.appendChild(current_document.createTextNode(this.label));
	
		tr_header.appendChild(td_header);

		var tr_data = current_document.createElement("tr");
		var td_data = current_document.createElement("td");
		td_data.appendChild(this.node);

		if(this.getType() == "HIDDEN") {
			td_data.appendChild(current_document.createTextNode(this.node.getAttribute("name") + " = " + this.node.getAttribute("value") ) );
		}

		tr_data.appendChild(td_data);

		table.appendChild(tr_header);
		table.appendChild(tr_data);

		tempNode.parentNode.replaceChild(table, tempNode);

	},

	unmark: function() {
		this.marked = false;

		/* get the original parent node (it's four levels up)*/
		var table = this.node.parentNode.parentNode.parentNode; /* table cell - table row - table */
		table.parentNode.replaceChild(this.node, table);

	},

	hasField : function(formField) {

		for(var i = 0; i < this.children.length; i++) {
			//formField.showAlert();
			//alert("looking for " + formField.id + " on " + this.children[i].getName() + ". Result: " + formField.equals(this.children[i].node));
			if(formField.equals(this.children[i].node)) return i;
		}
		return -1;
	},

	fillField : function(val, formField) {
		this.children[val].node.value = formField.value;
	},

	addNoteAround: function(title, text, color) {

		var markAgain = false;
		if(this.marked) {markAgain = true; this.unmark(); }

		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
		var current_document = browserWindow.contentDocument;

		/* replace node by temp node */
		var tempNode = current_document.createElement("tempNode");
		this.node.parentNode.replaceChild(tempNode, this.node);

		var new_container = gsCreateMarkContainer(this.node, "Test Notes (" + this.getName() + ")" , "orange", text);
		tempNode.parentNode.replaceChild(new_container, tempNode);

		if(markAgain) this.mark("red");
	},

	addNoteInside: function(title, text, color) {
		var new_container = gsCreateMarkContainer(null, "Note", color, text);
	}
}

/************************************************************************/
/* Actions object                                                       */
/************************************************************************/
/*  Used to execute actions in a form or a form element via the data    */
/*  model object                                                        */
/************************************************************************/

/*consts*/
GSACTION_NO_ACTION = 0;
GSACTION_ADD_ELEMENT = 1;
GSACTION_REMOVE_ELEMENT = 2;
GSACTION_ADD_ATTRIBUTE = 3;
GSACTION_REMOVE_ATTRIBUTE = 4;
GSACTION_CHANGE_ATTRIBUTE = 5;
GSACTION_CLONE_ELEMENT = 6;
GSACTION_REPLACE_ELEMENT = 7;


function GsAction(action, element, p) {
	this.type = action;
	this.targetElement = element;
	this.params = p;
}

GsAction.prototype = {
	type: this.NO_ACTION,
	targetElement: null,
	params: []
}


/************************************************************************/
/* Encoder object                                                       */
/************************************************************************/
/*  Used to encode, decode and hash data in form fields                 */
/*  Some code stolen from new TPS (thanks Josh!)                        */
/************************************************************************/

function GsEncodingToolbox() {

	this.base64Chars = new Array(
		'A','B','C','D','E','F','G','H',
		'I','J','K','L','M','N','O','P',
		'Q','R','S','T','U','V','W','X',
		'Y','Z','a','b','c','d','e','f',
		'g','h','i','j','k','l','m','n',
		'o','p','q','r','s','t','u','v',
		'w','x','y','z','0','1','2','3',
		'4','5','6','7','8','9','+','/'
	);
	
	for (var i=0; i < this.base64Chars.length; i++){
		this.reverseBase64Chars[this.base64Chars[i]] = i;
	}

}

GsEncodingToolbox.prototype = {

	END_OF_INPUT: -1,

	base64Chars: new Array(),

	reverseBase64Chars: new Array(),
	base64Str: null,
	base64Count: null,

	setBase64Str: function(str){
		this.base64Str = str;
		this.base64Count = 0;
	},

	readBase64: function(){    
		if (!this.base64Str) return this.END_OF_INPUT;
		if (this.base64Count >= this.base64Str.length) return this.END_OF_INPUT;
		var c = this.base64Str.charCodeAt(this.base64Count) & 0xff;
		this.base64Count++;
		return c;
	},

	encodeBase64: function(str){
		this.setBase64Str(str);
		var result = '';
		var inBuffer = new Array(3);
		var lineCount = 0;
		var done = false;
		while (!done && (inBuffer[0] = this.readBase64()) != this.END_OF_INPUT){
			inBuffer[1] = this.readBase64();
			inBuffer[2] = this.readBase64();
			result += (this.base64Chars[ inBuffer[0] >> 2 ]);
			if (inBuffer[1] != this.END_OF_INPUT){
				result += (this.base64Chars [(( inBuffer[0] << 4 ) & 0x30) | (inBuffer[1] >> 4) ]);
				if (inBuffer[2] != this.END_OF_INPUT){
					result += (this.base64Chars [((inBuffer[1] << 2) & 0x3c) | (inBuffer[2] >> 6) ]);
					result += (this.base64Chars [inBuffer[2] & 0x3F]);
				} else {
					result += (this.base64Chars [((inBuffer[1] << 2) & 0x3c)]);
					result += ('=');
					done = true;
				}
			} else {
				result += (this.base64Chars [(( inBuffer[0] << 4 ) & 0x30)]);
				result += ('=');
				result += ('=');
				done = true;
			}
			lineCount += 4;
			if (lineCount >= 76){
				result += ('\n');
				lineCount = 0;
			}
		}
		return result;
	},

	readReverseBase64: function(){   
		if (!this.base64Str) return this.END_OF_INPUT;
		while (true){      
			if (this.base64Count >= this.base64Str.length) return this.END_OF_INPUT;
			var nextCharacter = this.base64Str.charAt(this.base64Count);
			this.base64Count++;
			if (this.reverseBase64Chars[nextCharacter]){
				return this.reverseBase64Chars[nextCharacter];
			}
			if (nextCharacter == 'A') return 0;
			}
			return this.END_OF_INPUT;
	},

	ntos: function(n){
		n=n.toString(16);
		if (n.length == 1) n="0"+n;
		n="%"+n;
		return unescape(n);
	},

	decodeBase64: function(str){
		this.setBase64Str(str);
		var result = "";
		var inBuffer = new Array(4);
		var done = false;
		while (!done && (inBuffer[0] = this.readReverseBase64()) != this.END_OF_INPUT
			&& (inBuffer[1] = this.readReverseBase64()) != this.END_OF_INPUT){
			inBuffer[2] = this.readReverseBase64();
			inBuffer[3] = this.readReverseBase64();
			result += this.ntos((((inBuffer[0] << 2) & 0xff)| inBuffer[1] >> 4));
			if (inBuffer[2] != this.END_OF_INPUT){
				result +=  this.ntos((((inBuffer[1] << 4) & 0xff)| inBuffer[2] >> 2));
				if (inBuffer[3] != this.END_OF_INPUT){
					result +=  this.ntos((((inBuffer[2] << 6)  & 0xff) | inBuffer[3]));
				} else {
					done = true;
				}
			} else {
				done = true;
			}
		}
		return result;
	},

	digitArray: new Array('0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'),

	/* this function should be removed. It changes a charcode to hex string */
	toHex: function(n){
		var result = ''
		var start = true;
		for (var i=32; i>0;){
			i-=4;
			var digit = (n>>i) & 0xf;
			if (!start || digit != 0){
				start = false;
				result += this.digitArray[digit];
			}
		}
		return (result==''?'0':result);
	},

	encodeHex: function(data){
		function createIterator(x) {
			var i = 0;
			return function(){
				return x.charCodeAt(i++);
			};
		}
		var iterator = createIterator(data)
		current = 0;
		result = "";
		while(current = iterator()) {
			result += "%" + current.toString(16)
		}
		return result;
	},

	decodeHex: function(data){
		function createIterator(x) {
			var i = 0;
			return function(){
				pair = x.substring(i, i+2);
				if (i < data.length) { i += 2; }
				return pair;
			};
		}
		data = data.replace(/[^0-9a-fA-F]/g, "");
		iterator = createIterator(data);
		current = 0;
		result = "";
		while(current = iterator()) {
			result += String.fromCharCode(parseInt(current, 16))
		}
		return result
	},
		

/*
 * A JavaScript implementation of the RSA Data Security, Inc. MD5 Message
 * Digest Algorithm, as defined in RFC 1321.
 * Version 2.1 Copyright (C) Paul Johnston 1999 - 2002.
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for more info.
 */

/*
 * Configurable variables. You may need to tweak these to be compatible with
 * the server-side, but the defaults work in most cases.
 */
	hexcase: 0,  /* hex output format. 0 - lowercase; 1 - uppercase        */
	b64pad: "", /* base-64 pad character. "=" for strict RFC compliance   */
	chrsz: 8,  /* bits per input character. 8 - ASCII; 16 - Unicode      */

/*
 * These are the functions you'll usually want to call
 * They take string arguments and return either hex or base-64 encoded strings
 */

	hex_md5: function(s) { return this.binl2hex(this.core_md5(this.str2binl(s), s.length * this.chrsz));},
	b64_md5: function(s) { return this.binl2b64(this.core_md5(this.str2binl(s), s.length * this.chrsz));},
	str_md5: function(s) { return this.binl2str(this.core_md5(this.str2binl(s), s.length * this.chrsz));},
	hex_hmac_md5: function(key, data) { return this.binl2hex(this.core_hmac_md5(key, data)); },
	b64_hmac_md5: function(key, data) { return this.binl2b64(this.core_hmac_md5(key, data)); },
	str_hmac_md5: function(key, data) { return this.binl2str(this.core_hmac_md5(key, data)); },

/*
 * Perform a simple self-test to see if the VM is working
 */

	md5_vm_test: function() { return hex_md5("abc") == "900150983cd24fb0d6963f7d28e17f72"; },

/*
 * Calculate the MD5 of an array of little-endian words, and a bit length
 */
core_md5: function(x, len) {

  /* append padding */
  x[len >> 5] |= 0x80 << ((len) % 32);
  x[(((len + 64) >>> 9) << 4) + 14] = len;

  var a =  1732584193;
  var b = -271733879;
  var c = -1732584194;
  var d =  271733878;

  for(var i = 0; i < x.length; i += 16)
  {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;

    a = this.md5_ff(a, b, c, d, x[i+ 0], 7 , -680876936);
    d = this.md5_ff(d, a, b, c, x[i+ 1], 12, -389564586);
    c = this.md5_ff(c, d, a, b, x[i+ 2], 17,  606105819);
    b = this.md5_ff(b, c, d, a, x[i+ 3], 22, -1044525330);
    a = this.md5_ff(a, b, c, d, x[i+ 4], 7 , -176418897);
    d = this.md5_ff(d, a, b, c, x[i+ 5], 12,  1200080426);
    c = this.md5_ff(c, d, a, b, x[i+ 6], 17, -1473231341);
    b = this.md5_ff(b, c, d, a, x[i+ 7], 22, -45705983);
    a = this.md5_ff(a, b, c, d, x[i+ 8], 7 ,  1770035416);
    d = this.md5_ff(d, a, b, c, x[i+ 9], 12, -1958414417);
    c = this.md5_ff(c, d, a, b, x[i+10], 17, -42063);
    b = this.md5_ff(b, c, d, a, x[i+11], 22, -1990404162);
    a = this.md5_ff(a, b, c, d, x[i+12], 7 ,  1804603682);
    d = this.md5_ff(d, a, b, c, x[i+13], 12, -40341101);
    c = this.md5_ff(c, d, a, b, x[i+14], 17, -1502002290);
    b = this.md5_ff(b, c, d, a, x[i+15], 22,  1236535329);

    a = this.md5_gg(a, b, c, d, x[i+ 1], 5 , -165796510);
    d = this.md5_gg(d, a, b, c, x[i+ 6], 9 , -1069501632);
    c = this.md5_gg(c, d, a, b, x[i+11], 14,  643717713);
    b = this.md5_gg(b, c, d, a, x[i+ 0], 20, -373897302);
    a = this.md5_gg(a, b, c, d, x[i+ 5], 5 , -701558691);
    d = this.md5_gg(d, a, b, c, x[i+10], 9 ,  38016083);
    c = this.md5_gg(c, d, a, b, x[i+15], 14, -660478335);
    b = this.md5_gg(b, c, d, a, x[i+ 4], 20, -405537848);
    a = this.md5_gg(a, b, c, d, x[i+ 9], 5 ,  568446438);
    d = this.md5_gg(d, a, b, c, x[i+14], 9 , -1019803690);
    c = this.md5_gg(c, d, a, b, x[i+ 3], 14, -187363961);
    b = this.md5_gg(b, c, d, a, x[i+ 8], 20,  1163531501);
    a = this.md5_gg(a, b, c, d, x[i+13], 5 , -1444681467);
    d = this.md5_gg(d, a, b, c, x[i+ 2], 9 , -51403784);
    c = this.md5_gg(c, d, a, b, x[i+ 7], 14,  1735328473);
    b = this.md5_gg(b, c, d, a, x[i+12], 20, -1926607734);

    a = this.md5_hh(a, b, c, d, x[i+ 5], 4 , -378558);
    d = this.md5_hh(d, a, b, c, x[i+ 8], 11, -2022574463);
    c = this.md5_hh(c, d, a, b, x[i+11], 16,  1839030562);
    b = this.md5_hh(b, c, d, a, x[i+14], 23, -35309556);
    a = this.md5_hh(a, b, c, d, x[i+ 1], 4 , -1530992060);
    d = this.md5_hh(d, a, b, c, x[i+ 4], 11,  1272893353);
    c = this.md5_hh(c, d, a, b, x[i+ 7], 16, -155497632);
    b = this.md5_hh(b, c, d, a, x[i+10], 23, -1094730640);
    a = this.md5_hh(a, b, c, d, x[i+13], 4 ,  681279174);
    d = this.md5_hh(d, a, b, c, x[i+ 0], 11, -358537222);
    c = this.md5_hh(c, d, a, b, x[i+ 3], 16, -722521979);
    b = this.md5_hh(b, c, d, a, x[i+ 6], 23,  76029189);
    a = this.md5_hh(a, b, c, d, x[i+ 9], 4 , -640364487);
    d = this.md5_hh(d, a, b, c, x[i+12], 11, -421815835);
    c = this.md5_hh(c, d, a, b, x[i+15], 16,  530742520);
    b = this.md5_hh(b, c, d, a, x[i+ 2], 23, -995338651);

    a = this.md5_ii(a, b, c, d, x[i+ 0], 6 , -198630844);
    d = this.md5_ii(d, a, b, c, x[i+ 7], 10,  1126891415);
    c = this.md5_ii(c, d, a, b, x[i+14], 15, -1416354905);
    b = this.md5_ii(b, c, d, a, x[i+ 5], 21, -57434055);
    a = this.md5_ii(a, b, c, d, x[i+12], 6 ,  1700485571);
    d = this.md5_ii(d, a, b, c, x[i+ 3], 10, -1894986606);
    c = this.md5_ii(c, d, a, b, x[i+10], 15, -1051523);
    b = this.md5_ii(b, c, d, a, x[i+ 1], 21, -2054922799);
    a = this.md5_ii(a, b, c, d, x[i+ 8], 6 ,  1873313359);
    d = this.md5_ii(d, a, b, c, x[i+15], 10, -30611744);
    c = this.md5_ii(c, d, a, b, x[i+ 6], 15, -1560198380);
    b = this.md5_ii(b, c, d, a, x[i+13], 21,  1309151649);
    a = this.md5_ii(a, b, c, d, x[i+ 4], 6 , -145523070);
    d = this.md5_ii(d, a, b, c, x[i+11], 10, -1120210379);
    c = this.md5_ii(c, d, a, b, x[i+ 2], 15,  718787259);
    b = this.md5_ii(b, c, d, a, x[i+ 9], 21, -343485551);

    a = this.safe_add(a, olda);
    b = this.safe_add(b, oldb);
    c = this.safe_add(c, oldc);
    d = this.safe_add(d, oldd);
  }
  return Array(a, b, c, d);
},


/*
 * These functions implement the four basic operations the algorithm uses.
 */
md5_cmn: function(q, a, b, x, s, t)
{
  return this.safe_add(this.bit_rol(this.safe_add(this.safe_add(a, q), this.safe_add(x, t)), s),b);
},
md5_ff: function(a, b, c, d, x, s, t)
{
  return this.md5_cmn((b & c) | ((~b) & d), a, b, x, s, t);
},
md5_gg: function(a, b, c, d, x, s, t)
{
  return this.md5_cmn((b & d) | (c & (~d)), a, b, x, s, t);
},
md5_hh: function(a, b, c, d, x, s, t)
{
  return this.md5_cmn(b ^ c ^ d, a, b, x, s, t);
},
md5_ii: function(a, b, c, d, x, s, t)
{
  return this.md5_cmn(c ^ (b | (~d)), a, b, x, s, t);
},

/*
 * Calculate the HMAC-MD5, of a key and some data
 */
core_hmac_md5: function(key, data)
{
  var bkey = this.str2binl(key);
  if(bkey.length > 16) bkey = this.core_md5(bkey, key.length * chrsz);

  var ipad = Array(16), opad = Array(16);
  for(var i = 0; i < 16; i++)
  {
    ipad[i] = bkey[i] ^ 0x36363636;
    opad[i] = bkey[i] ^ 0x5C5C5C5C;
  }

  var hash = this.core_md5(ipad.concat(this.str2binl(data)), 512 + data.length * this.chrsz);
  return this.core_md5(opad.concat(hash), 512 + 128);
},

/*
 * Add integers, wrapping at 2^32. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 */
safe_add: function(x, y)
{
  var lsw = (x & 0xFFFF) + (y & 0xFFFF);
  var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return (msw << 16) | (lsw & 0xFFFF);
},

/*
 * Bitwise rotate a 32-bit number to the left.
 */
bit_rol: function(num, cnt)
{
  return (num << cnt) | (num >>> (32 - cnt));
},

/*
 * Convert a string to an array of little-endian words
 * If chrsz is ASCII, characters >255 have their hi-byte silently ignored.
 */
str2binl: function(str)
{
  var bin = Array();
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < str.length * this.chrsz; i += this.chrsz)
    bin[i>>5] |= (str.charCodeAt(i / this.chrsz) & mask) << (i%32);
  return bin;
},

/*
 * Convert an array of little-endian words to a string
 */
binl2str: function(bin)
{
  var str = "";
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < bin.length * 32; i += this.chrsz)
    str += String.fromCharCode((bin[i>>5] >>> (i % 32)) & mask);
  return str;
},


/*
 * Convert an array of little-endian words to a hex string.
 */
binl2hex: function(binarray)
{
  var hex_tab = this.hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i++)
  {
    str += hex_tab.charAt((binarray[i>>2] >> ((i%4)*8+4)) & 0xF) +
           hex_tab.charAt((binarray[i>>2] >> ((i%4)*8  )) & 0xF);
  }
  return str;
},

/*
 * Convert an array of little-endian words to a base-64 string
 */
binl2b64: function(binarray)
{
  var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i += 3)
  {
    var triplet = (((binarray[i   >> 2] >> 8 * ( i   %4)) & 0xFF) << 16)
                | (((binarray[i+1 >> 2] >> 8 * ((i+1)%4)) & 0xFF) << 8 )
                |  ((binarray[i+2 >> 2] >> 8 * ((i+2)%4)) & 0xFF);
    for(var j = 0; j < 4; j++)
    {
      if(i * 8 + j * 6 > binarray.length * 32) str += this.b64pad;
      else str += tab.charAt((triplet >> 6*(3-j)) & 0x3F);
    }
  }
  return str;
},


/*
 * A JavaScript implementation of the Secure Hash Algorithm, SHA-1, as defined
 * in FIPS PUB 180-1
 * Version 2.1a Copyright Paul Johnston 2000 - 2002.
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for details.
 */

/*
 * These are the functions you'll usually want to call
 * They take string arguments and return either hex or base-64 encoded strings
 */
	hex_sha1: function(s){return this.binb2hex(this.core_sha1(this.str2binb(s),s.length * this.chrsz));},
	b64_sha1: function(s){return this.binb2b64(this.core_sha1(this.str2binb(s),s.length * this.chrsz));},
	str_sha1: function(s){return this.binb2str(this.core_sha1(this.str2binb(s),s.length * this.chrsz));},
	hex_hmac_sha1: function(key, data){ return this.binb2hex(this.core_hmac_sha1(key, data));},
	b64_hmac_sha1: function(key, data){ return this.binb2b64(this.core_hmac_sha1(key, data));},
	str_hmac_sha1: function(key, data){ return this.binb2str(this.core_hmac_sha1(key, data));},

/*
 * Perform a simple self-test to see if the VM is working
 */
	sha1_vm_test: function() { return hex_sha1("abc") == "a9993e364706816aba3e25717850c26c9cd0d89d"; },

/*
 * Calculate the SHA-1 of an array of big-endian words, and a bit length
 */
core_sha1: function(x, len)
{
  /* append padding */
  x[len >> 5] |= 0x80 << (24 - len % 32);
  x[((len + 64 >> 9) << 4) + 15] = len;

  var w = Array(80);
  var a =  1732584193;
  var b = -271733879;
  var c = -1732584194;
  var d =  271733878;
  var e = -1009589776;

  for(var i = 0; i < x.length; i += 16)
  {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;
    var olde = e;

    for(var j = 0; j < 80; j++)
    {
      if(j < 16) w[j] = x[i + j];
      else w[j] = this.bit_rol(w[j-3] ^ w[j-8] ^ w[j-14] ^ w[j-16], 1);
      var t = this.safe_add(this.safe_add(this.bit_rol(a, 5), this.sha1_ft(j, b, c, d)),
                       this.safe_add(this.safe_add(e, w[j]), this.sha1_kt(j)));
      e = d;
      d = c;
      c = this.bit_rol(b, 30);
      b = a;
      a = t;
    }

    a = this.safe_add(a, olda);
    b = this.safe_add(b, oldb);
    c = this.safe_add(c, oldc);
    d = this.safe_add(d, oldd);
    e = this.safe_add(e, olde);
  }
  return Array(a, b, c, d, e);

},


/*
 * Perform the appropriate triplet combination function for the current
 * iteration
 */
sha1_ft: function(t, b, c, d)
{
  if(t < 20) return (b & c) | ((~b) & d);
  if(t < 40) return b ^ c ^ d;
  if(t < 60) return (b & c) | (b & d) | (c & d);
  return b ^ c ^ d;
},

/*
 * Determine the appropriate additive constant for the current iteration
 */
sha1_kt: function(t)
{
  return (t < 20) ?  1518500249 : (t < 40) ?  1859775393 :
         (t < 60) ? -1894007588 : -899497514;
},

/*
 * Calculate the HMAC-SHA1 of a key and some data
 */
core_hmac_sha1: function(key, data)
{
  var bkey = this.str2binb(key);
  if(bkey.length > 16) bkey = this.core_sha1(bkey, key.length * this.chrsz);

  var ipad = Array(16), opad = Array(16);
  for(var i = 0; i < 16; i++)
  {
    ipad[i] = bkey[i] ^ 0x36363636;
    opad[i] = bkey[i] ^ 0x5C5C5C5C;
  }

  var hash = this.core_sha1(ipad.concat(this.str2binb(data)), 512 + data.length * this.chrsz);
  return this.core_sha1(opad.concat(hash), 512 + 160);
},

/*
 * Convert an 8-bit or 16-bit string to an array of big-endian words
 * In 8-bit function, characters >255 have their hi-byte silently ignored.
 */
str2binb: function(str)
{
  var bin = Array();
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < str.length * this.chrsz; i += this.chrsz)
    bin[i>>5] |= (str.charCodeAt(i / this.chrsz) & mask) << (32 - this.chrsz - i%32);
  return bin;
},

/*
 * Convert an array of big-endian words to a string
 */
binb2str: function(bin)
{
  var str = "";
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < bin.length * 32; i += this.chrsz)
    str += String.fromCharCode((bin[i>>5] >>> (32 - this.chrsz - i%32)) & mask);
  return str;
},

/*
 * Convert an array of big-endian words to a hex string.
 */
binb2hex: function(binarray)
{
  var hex_tab = this.hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i++)
  {
    str += hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8+4)) & 0xF) +
           hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8  )) & 0xF);
  }
  return str;
},

/*
 * Convert an array of big-endian words to a base-64 string
 */
binb2b64: function(binarray)
{
  var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i += 3)
  {
    var triplet = (((binarray[i   >> 2] >> 8 * (3 -  i   %4)) & 0xFF) << 16)
                | (((binarray[i+1 >> 2] >> 8 * (3 - (i+1)%4)) & 0xFF) << 8 )
                |  ((binarray[i+2 >> 2] >> 8 * (3 - (i+2)%4)) & 0xFF);
    for(var j = 0; j < 4; j++)
    {
      if(i * 8 + j * 6 > binarray.length * 32) str += this.b64pad;
      else str += tab.charAt((triplet >> 6*(3-j)) & 0x3F);
    }
  }
  return str;
},

	/* Code for HTML numeric entities encoding */

	htmlNumEntEncode: function(val, text) {
		var result = '';
		for (i=0; i<text.length; i++) {
			switch(val) {
				case 0: result += '&#' + this.toHex(text.charCodeAt(i)) + ';'; break;
				case 1: result += '&#' + text.charCodeAt(i) + ';'; break;
			}
		}
		return result;
	},

	/* Code for HTML entities encoding */

	htmlEntEncode: function(text) {


		/* not yet used

		names2entities["&copy"] = "&#169;";
		names2entities["&reg"] = "&#174;";
		names2entities["&sup2"] = "&#178;";
		names2entities["&sup3"] = "&#179;";
		names2entities["&quot"] = "&#34;";
		names2entities["&amp"] = "&#38;";
		names2entities["&lt"] = "&#60;";
		names2entities["&gt"] = "&#62;";
		names2entities["&ndash"] = "&#8211;";
		names2entities["&mdash"] = "&#8212;";
		names2entities["&lsquo"] = "&#8216;";
		names2entities["&rsquo"] = "&#8217;";
		names2entities["&ldquo"] = "&#8220;";
		names2entities["&rdquo"] = "&#8221;";
		names2entities["&bull"] = "&#8226;";
		names2entities["&dagger"] = "&#8224;";
		names2entities["&Dagger"] = "&#8225;";
		names2entities["&prime"] = "&#8242;";
		names2entities["&Prime"] = "&#8243;";
		names2entities["&lsaquo"] = "&#8249;";
		names2entities["&rsaquo"] = "&#8250;";
		names2entities["&euro"] = "&#8364;";
		names2entities["&trade"] = "&#8482;";
		names2entities["&tilde"] = "&#732;";
		names2entities["&circ"] = "&#710;";
		names2entities["&spades"] = "&#9824;";
		names2entities["&clubs"] = "&#9827;";
		names2entities["&hearts"] = "&#9829;";
		names2entities["&diams"] = "&#9830;";
		names2entities["&loz"] = " &#9674;";
		names2entities["&larr"] = "&#8592;";
		names2entities["&rarr"] = "&#8594;";
		names2entities["&uarr"] = "&#8593;";
		names2entities["&darr"] = "&#8595;";
		names2entities["&harr"] = "&#8596;";
		names2entities["&not"] = "&#172;";

		*/

		var entities2names = [];

		entities2names["&#169;"] = "&copy; ";
		entities2names["&#174;"] = "&reg;";
		entities2names["&#178;"] = "&sup2;";
		entities2names["&#179;"] = "&sup3;";
		entities2names["&#34;"] = "&quot;";
		entities2names["&#38;"] = "&amp;";
		entities2names["&#60;"] = "&lt;";
		entities2names["&#62;"] = "&gt;";
		entities2names["&#8211;"] = "&ndash;";
		entities2names["&#8212;"] = "&mdash;";
		entities2names["&#8216;"] = "&lsquo;";
		entities2names["&#8217;"] = "&rsquo;";
		entities2names["&#8220;"] = "&ldquo;";
		entities2names["&#8221;"] = "&rdquo;";
		entities2names["&#8226;"] = "&bull;";
		entities2names["&#8224;"] = "&dagger;";
		entities2names["&#8225;"] = "&Dagger;";
		entities2names["&#8242;"] = "&prime;";
		entities2names["&#8243;"] = "&Prime;";
		entities2names["&#8249;"] = "&lsaquo;";
		entities2names["&#8250;"] = "&rsaquo;";
		entities2names["&#8364;"] = "&euro;";
		entities2names["&#8482;"] = "&trade;";
		entities2names["&#732;"] = "&tilde;";
		entities2names["&#710;"] = "&circ;";
		entities2names["&#9824;"] = "&spades;";
		entities2names["&#9827;"] = "&clubs;";
		entities2names["&#9829;"] = "&hearts;";
		entities2names["&#9830;"] = "&diams;";
		entities2names["&#9674;"] = "&loz;";
		entities2names["&#8592;"] = "&larr;";
		entities2names["&#8594;"] = "&rarr;";
		entities2names["&#8593;"] = "&uarr;";
		entities2names["&#8595;"] = "&darr;";
		entities2names["&#8596;"] = "&harr;";
		entities2names["&#172;"] = "&not;";


		var result = '';
		var num = "";
		for (i=0; i<text.length; i++) {
			num = '&#' + text.charCodeAt(i) + ';';
			if(entities2names[num]) {
				result += entities2names[num];
			}
			else {
				result += text.charAt(i);
			}
		}
		return result;

	},

	encodeUnicode: function(val, str){
		var result = "";
		var codePrefix = "";
		switch(val) {
			case 0: codePrefix = "\\u"; break;
			case 1: codePrefix = "U+"; break;
		}
		for (var i=0; i<str.length; i++){
			var prefix = "";
			var hex = this.toHex(str.charCodeAt(i));
			for(var j=hex.length; j < 4; j++) { prefix += "0"; }
			
			result += codePrefix + prefix + hex;
		}
		return result;
	},

	encodeHexEscaped: function(str){
		var result = "";
		for (var i=0; i<str.length; i++){
			var hex = this.toHex(str.charCodeAt(i));
			result += "\\x" + hex;
		}
		return result;
	},


} /* end of GsEncodingToolbox prototype */


/************************************************************************/
/************************************************************************/
/* INIT CODE and GLOBAL VARS                                            */
/************************************************************************/
/************************************************************************/

function gsTreeInit() {
	document.getElementById("gsFormElementListBox").view = gsDataModel;
}


/* Create the global var that will hold the Data Model object */
var gsDataModel = new GsDataModel();


/************************************************************************/
/************************************************************************/
/* FUNCTIONS DECLARATIONS                                               */
/************************************************************************/
/************************************************************************/


/************************************************************************/
/* Functions used for the context menu                                  */
/************************************************************************/

function updateItemContextMenu() {


	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* Get the context menu */
	var menu = document.getElementById("gsFormItemContextMenu");

	/* Clean up whatever is currently in the menu */
	for(var i = menu.childNodes.length - 1; i>=0; i--) {
		menu.removeChild(menu.childNodes.item(i));
	}

	var options = [];
	options["FORM"] = [
		["id", "gsAddNewElement", "label", getLocaleString("menu.addNewElement"), "oncommand", "gsAddNewElementMenuOption()" ],
		["id", "gsAddNewSubmit", "label", getLocaleString("menu.addNewSubmit"), "oncommand", "gsAddNewSubmitMenuOption()"],
		["id", "gsAddNewInputMenuOption", "label", getLocaleString("menu.addNewTextField"), "oncommand", "gsAddNewInputMenuOption('text')"],
		["id", "gsSetTargetToNewWindow", "label", getLocaleString("menu.submitInNewWindow"), "oncommand", "gsSetFormTargetToNewWindow()"],
		["id", "gsChangeFormMethod", "label", getLocaleString("menu.changeFormMethod"), "oncommand", "gsChangeFormMethod()"],
		["id", "gsChangeAllHiddenToText", "label", getLocaleString("menu.changeAllHiddenToText"), "oncommand", "gsChangeAllHiddenToText()"]
	];
	options["SAVE"] = [
		["id", "gsSaveFormValues", "label", getLocaleString("menu.saveFormValues"), "oncommand", "gsSaveFormValues()"],
		["id", "gsClearSavedFormSets", "label", getLocaleString("menu.clearListSavedForms"), "oncommand", "gsClearSavedFormSets()"]
	];
	options["ALL"] = [
		/* ["id", "gsAddTextNote", "label", "Add a Text Note", "oncommand", "gsAddTextNote()"], // This is for next version! */
		["id", "gsRemoveFormElement", "label", getLocaleString("menu.removeElement"), "oncommand", "gsRemoveFormElement()"],
		["id", "gsRemoveAllEventHandlers", "label", getLocaleString("menu.removeAllEventHandlers"), "oncommand", "gsRemoveAllEventHandlers()"]
	];
	options["NOT_TEXT"] = [
		["id", "gsTransformIntoTextInput", "label", getLocaleString("menu.transformIntoTextInput"), "oncommand", "gsChangeToText()"]
	];
	options["TEXT"] = [
		["id", "gsRemoveLengthLimits", "label", getLocaleString("menu.removeLengthLimits"), "oncommand", "gsRemoveLengthLimits()"],
		["id", "gsModifyInputSize", "label", getLocaleString("menu.modifySize"), "oncommand", "gsModifyInputSize()"]
	];
	options["BASE64"] = [
		["id", "gsBase64EncodeText", "label", getLocaleString("menu.base64Encode"), "oncommand", "gsBase64EncodeText()"],
		["id", "gsBase64DecodeText", "label", getLocaleString("menu.base64Decode"), "oncommand", "gsBase64DecodeText()"]
	];
	options["HEX"] = [
		["id", "gsHexEncodeText", "label", getLocaleString("menu.hexEncodePercent"), "oncommand", "gsHexEncodeText()"],
		["id", "gsHexEscapedEncode", "label", getLocaleString("menu.hexEncodeSlashes"), "oncommand", "gsHexEscapedEncode()"],
		["id", "gsHexDecodeText", "label", getLocaleString("menu.hexDecode"), "oncommand", "gsHexDecodeText()"],
	];
	options["MD5"] = [
		["id", "gsHexMd5", "label", getLocaleString("menu.md5HashHex"), "oncommand", "gsHexMd5()"],
		["id", "gsB64Md5", "label", getLocaleString("menu.md5HashBase64"), "oncommand", "gsB64Md5()"],
		["id", "gsStrMd5", "label", getLocaleString("menu.md5HashString"), "oncommand", "gsStrMd5()"],
	];
	options["SHA1"] = [
		["id", "gsHexSha1", "label", getLocaleString("menu.sha1HashHex"), "oncommand", "gsHexSha1()"],
		["id", "gsB64Sha1", "label", getLocaleString("menu.sha1HashBase64"), "oncommand", "gsB64Sha1()"],
		["id", "gsStrSha1", "label", getLocaleString("menu.sha1HashString"), "oncommand", "gsStrSha1()"],
	];
	options["HTMLENT"] = [
		["id", "gsHtlmEnt", "label", getLocaleString("menu.entitiesEncode"), "oncommand", "gsHtlmEnt()"],
		["id", "gsHtlmNumEnt", "label", getLocaleString("menu.entitiesEncodeHex"), "oncommand", "gsHtlmNumEnt(0)"],
		["id", "gsHtlmNumEnt", "label", getLocaleString("menu.entitiesEncodeDec"), "oncommand", "gsHtlmNumEnt(1)"],
	];
	options["UNICODE"] = [
		["id", "gsUnicodeEncode", "label", getLocaleString("menu.unicodeEncodeSlashes"), "oncommand", "gsUnicodeEncode(0)"],
		["id", "gsUnicodeEncode", "label", getLocaleString("menu.unicodeEncodeBigU"), "oncommand", "gsUnicodeEncode(1)"],
	];
	options["URLENCODE"] = [
		["id", "gsURLEncode", "label", getLocaleString("menu.urlEncode"), "oncommand", "gsURLEncode()"],
		["id", "gsURLDecode", "label", getLocaleString("menu.urlDecode"), "oncommand", "gsURLDecode()"],
	];
	options["CHARCASE"] = [
		["id", "gsToUpperCase", "label", getLocaleString("menu.toUpper"), "oncommand", "gsToUpperCase()"],
		["id", "gsToLowerCase", "label", getLocaleString("menu.toLower"), "oncommand", "gsToLowerCase()"],
	];


	/* options that are displayed for all elements */ 
	if((element.getType() != "TEXT") && (element.getType() != "FORM")) {
		gsPopulateDynamicMenuItems(menu, options["NOT_TEXT"]);
		menu.appendChild(document.createElement("menuseparator"));
	}

	switch(element.getType()) {
		case "FORM":
			gsPopulateDynamicMenuItems(menu, options["FORM"]);
			menu.appendChild(document.createElement("menuseparator"));
			gsPopulateDynamicMenuItems(menu, options["SAVE"]);
			var temp = [];
			for(var i = 0; i < gsDataModel.savedFormValues.length; i++) {
				var savedSet = gsDataModel.savedFormValues[i];
				temp.push(["id", "gsLoadSet" + savedSet[0] + i, "label", getLocaleString("loadFormValuesFrom") + " \" " + savedSet[0] + " \"", "oncommand", "gsLoadFormValues(" + selectedItem + ", " + i + ")"]);
			}
			gsPopulateDynamicMenuItems(menu, temp);
			menu.appendChild(document.createElement("menuseparator"));
			break;
		case "TEXT":
			gsPopulateDynamicMenuItems(menu, options["TEXT"]);
			menu.appendChild(document.createElement("menuseparator"));
		case "TEXTAREA":
		case "HIDDEN":
			gsPopulateDynamicMenuItems(menu, options["BASE64"]);
			gsPopulateDynamicMenuItems(menu, options["HEX"]);
			gsPopulateDynamicMenuItems(menu, options["HTMLENT"]);
			gsPopulateDynamicMenuItems(menu, options["UNICODE"]);
			gsPopulateDynamicMenuItems(menu, options["URLENCODE"]);
			menu.appendChild(document.createElement("menuseparator"));
			gsPopulateDynamicMenuItems(menu, options["CHARCASE"]);
			menu.appendChild(document.createElement("menuseparator"));
			gsPopulateDynamicMenuItems(menu, options["MD5"]);
			gsPopulateDynamicMenuItems(menu, options["SHA1"]);
			menu.appendChild(document.createElement("menuseparator"));
			break;
	}

	gsPopulateDynamicMenuItems(menu, options["ALL"]);

}

function gsCreateAndAddSubMenu(menu, label, id, data) {

	var submenu = document.createElement("menu");
	submenu.setAttribute("label", label);
	submenu.setAttribute("id", id);	

	var submenupopup = document.createElement("menupopup");
	submenupopup.setAttribute("id", id + "_menupopup");
	
	gsPopulateDynamicMenuItems(submenupopup, data);
	
	submenu.appendChild(submenupopup);
	menu.appendChild(submenu);
}


function gsPopulateDynamicMenuItems(menu, data){
	if(data.length > 0) {
		var attname = "";
		var attvalue = "";
	for(var j = 0; j < data.length; j++) {
			var new_element = document.createElement("menuitem");
			var itemData = data[j];
			for(var i = 0; i < itemData.length; i+=2) {
				attname = itemData[i];
				if(itemData[i+1]) attvalue = itemData[i+1];
				else attvalue = "";
				new_element.setAttribute(attname, attvalue);
				//alert(attname + "=" + attvalue);
			}
			menu.appendChild(new_element);
		}
	}
}

/************************************************************************/
/* Functions used for creating mark tables                              */
/************************************************************************/

function gsCreateMarkContainer(element, text, color, moreData) {
	var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
	var current_document = browserWindow.contentDocument;

	var bgcolor2 = "yellow";
	switch (color) {
		case "red": bgcolor2 = "#FFD3DA"; break;
		case "blue": bgcolor2 = "#DDE2FE"; break;
		case "darkgreen": bgcolor2 = "#DBFFD4" ; break;
		case "orange": bgcolor2 = "#FFFAA6"; break;
		default: bgcolor = "silver";
	}

	/* create container element */
	var table = current_document.createElement("table");
	table.setAttribute("style", "border: 1px solid " + color + "; font-family: sans-serif; font-size: small; padding: 0px; margin: 4px");

	var tr_header = current_document.createElement("tr");
	tr_header.setAttribute("style", "padding:0px;");
	var td_header = current_document.createElement("td");
	td_header.setAttribute("style", "background-color: " + color + "; color: white; font-weight: bold; padding: 2px 5px; margin: 0px");

	td_header.appendChild(current_document.createTextNode(text));
	tr_header.appendChild(td_header);

	var tr_data = current_document.createElement("tr");
	var td_data = current_document.createElement("td");
	td_data.setAttribute("style", "background: " + bgcolor2 + "; text-align: center");
	td_data.appendChild(element);

	if(moreData.tagName) { /* this means it is a node and not a string, append the node */
		td_data.appendChild(moreData);
	}
	else { /* else, it's a string, create a text node for it and append*/
		td_data.appendChild(current_document.createElement("br"));
		td_data.appendChild(current_document.createTextNode(moreData));
	}
	
	tr_data.appendChild(td_data);

	table.appendChild(tr_header);
	table.appendChild(tr_data);

	return table;
}

/************************************************************************/
/* Functions used to updated the GUI                                    */
/************************************************************************/

function gsSelectFormInterestingTags(aNode) {

	//alert(aNode.tagName + " => " +aNode.tagName.toUpperCase());
	
	const nsIInputElement    = Components.interfaces.nsIDOMHTMLInputElement;
	const nsIFormElement     = Components.interfaces.nsIDOMHTMLFormElement;
	const nsIButtonElement   = Components.interfaces.nsIDOMHTMLButtonElement;
	const nsISelectElement   = Components.interfaces.nsIDOMHTMLSelectElement;
	const nsITextareaElement = Components.interfaces.nsIDOMHTMLTextAreaElement;

	if ( (aNode instanceof nsIInputElement) || (aNode.tagName.toUpperCase() == "INPUT") || (aNode instanceof nsIFormElement) || (aNode instanceof nsIButtonElement) || (aNode instanceof nsISelectElement) || (aNode instanceof nsITextareaElement) ) { return NodeFilter.FILTER_ACCEPT; }
	else { return NodeFilter.FILTER_SKIP; }

}

function gsFillFormAttributes(box, node) {

	while(box.getRowCount() != 0) {
		box.removeItemAt(0);
	}

	for(var i = 0; node.attributes.length > i; i++) {
		var attr = node.attributes[i];
		var label;

		if(attr.name == "value") {
			label = attr.name + " = " + node.value;
		}
		else {
			label = attr.name + " = " + attr.value;
		}

		var listItem = box.appendItem(label, attr.name);
	}

}

function gsGetFrames(elem) {

	if(elem.tagName.toUpperCase() == "FRAME") return NodeFilter.FILTER_ACCEPT;
	else return NodeFilter.FILTER_SKIP;

}

function gsGetAllFrames(elem) {
		
	var iterator = document.createTreeWalker(elem, NodeFilter.SHOW_ELEMENT, gsGetFrames, true); 
	var frame = iterator.nextNode();
	while (frame) {

		//alert("found a frame: " + frame.name);
		var forms = frame.contentDocument.forms;

		/* for each form, add the form to the data model */
		for(var i = 0; forms.length > i; i++) {
			var form = gsDataModel.addForm(forms[i]);
		}

		/* call gsGetAllFrames() for each frame */
		gsGetAllFrames(frame.contentDocument);
		
		frame = iterator.nextNode();
	}

}

function gsUpdateFormElementListBox() {

	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	if(document.getElementById("gsFormElementListBox").hidden) return;

	gsDataModel.unmark();
	gsDataModel.reset();


	/* clear form item listbox */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	gsFillFormAttributes(gsFormItemsListBox, document.createElement("foo"));

	var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
	var current_document = browserWindow.contentDocument;

	if(current_document.forms.length > 0) { /* if the main document has forms */
		var forms = current_document.forms;
		for(var i = 0; forms.length > i; i++) {
			var form = gsDataModel.addForm(forms[i]);
		}
	}
	else { 
	}

	/* now look for frames... and forms inside them */
	gsGetAllFrames(current_document);
	
	gsDataModel.repaint();

}

/************************************************************************/
/* XPATH searches (Code from http://kb.mozillazine.org/XPath)           */
/************************************************************************/

/* BEGIN - Code from http://kb.mozillazine.org/XPath */
function gsEvaluateXPath(aNode, aExpr) {

  var xpe = new XPathEvaluator();
  var nsResolver = xpe.createNSResolver(aNode.ownerDocument == null ?
    aNode.documentElement : aNode.ownerDocument.documentElement);
  var result = xpe.evaluate(aExpr, aNode, nsResolver, 0, null);
  var found = [];
  var res;
  while (res = result.iterateNext())
    found.push(res);
  return found;
}
/* END - Code from http://kb.mozillazine.org/XPath */



/************************************************************************/
/* Functions used to handle events from the GUI                         */
/************************************************************************/

function gsClickOnFormElement(aEvent) {

	if(aEvent.type == "click") {
		if(/*Event.button == 2 ||*/ aEvent.originalTarget.localName != "treechildren") {
			return;
		}
	} else if(aEvent.type == "keypress") {
		if(aEvent.originalTarget.localName != "treechildren") {
			return;
		}
	}

	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");
	
	/* this is what need to change - get selection from tree? */
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	if( selectedItem >=0 ) gsDataModel.mark(selectedItem, "red");

	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

/************************************************************************/
/* Functions used to handle events in the attributes listbox            */
/************************************************************************/

function gsClickOnAttribute(aEvent) {
	if(aEvent.type == "click") {
		if(aEvent.button == 2 || aEvent.originalTarget.localName != "listitem") {
			return;
		}
		if(aEvent.detail < 2) {
			return;
		}
	} else if(aEvent.type == "keypress") {
		if(aEvent.originalTarget.localName != "listbox") {
			return;
		}
	}

	gsEditAttribute();
}


function gsEditAttributeMenuOption() {
	gsEditAttribute();
}


function gsEditAttribute() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute name */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");
	var attrItem = gsFormItemsListBox.selectedItem;
	var attribute = attrItem.value;

	/* prompt user for a new value */

	var newValue = prompt(getLocaleString("setNewValueFor") + " " + element.label, element.node.getAttribute(attribute));

	if(newValue) {
		/* create params array */
		var param = [attribute, newValue];

		/* execute action */
		var action = new GsAction(GSACTION_CHANGE_ATTRIBUTE, element, param);
		gsDataModel.performAction(action);

		/* refresh attribute box */
		gsFillFormAttributes(gsFormItemsListBox, element.node);
	}

	/* update form tree */
	boxobject = gsDataModel.treeBox;
	boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
	boxobject.rowCountChanged(selectedItem, 1);
}

function gsRemoveAttributeMenuOption() {
	//alert("remove in context menu");
	gsRemoveAttribute();
}

function gsRemoveAttribute() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute name */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");
	var attrItem = gsFormItemsListBox.selectedItem;
	var attribute = attrItem.value;

	/* create params array */
	var param = [attribute];

	/* execute action */
	var action = new GsAction(GSACTION_REMOVE_ATTRIBUTE, element, param);
	gsDataModel.performAction(action);

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsAddAttributeMenuOption() {
	gsAddAttribute();
}

function gsAddAttribute() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var attribute = prompt(getLocaleString("enterAttributeName"),"");
	if(attribute) {
		var value = prompt(getLocaleString("enterAttributeValue") + " \""+ attribute +"\"","");
		if(value){

			/* create params array */
			var param = [attribute, value];

			/* execute action */
			var action = new GsAction(GSACTION_ADD_ATTRIBUTE, element, param);
			gsDataModel.performAction(action);

			/* refresh attribute box */
			gsFillFormAttributes(gsFormItemsListBox, element.node);
		}
	}
}



/************************************************************************/
/* Functions used to handle events in the forms tree                    */
/************************************************************************/

function gsAddNewElementMenuOption() {
	gsAddNewElement();
}

function gsAddNewElement() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	if(element.getType() != "FORM") return;

	var tag = prompt(getLocaleString("enterTag") + " " + element.label,"");
	if(tag) {
		var name = prompt(getLocaleString("enterTagName1")+ " " + tag.toUpperCase() + " " + getLocaleString("enterTagName2") + " " + element.label,"");

		if(name) {
			/* create params array */
			var param = [tag, name];

			/* execute action */
			var action = new GsAction(GSACTION_ADD_ELEMENT, element, param);
			gsDataModel.performAction(action);
		}
	}

	gsUpdateFormElementListBox();
}

function gsCreateElement(element, tag, name, attrs){
	/* create params array */
	var params = [tag, name];
	params = params.concat(attrs);

	/* execute create action */
	var action = new GsAction(GSACTION_ADD_ELEMENT, element, params);
	gsDataModel.performAction(action);
}

function gsAddNewSubmitMenuOption() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	if(element.getType() != "FORM") return;

	gsCreateElement(element, "input", "submitButton", ["type", "submit", "value", getLocaleString("submit")]);

	//gsUpdateFormElementListBox();
}

function gsAddNewInputMenuOption(type) {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	if(element.getType() != "FORM") return;

	var name = prompt(getLocaleString("enterTagName1") + " " + type.toUpperCase() + " " + getLocaleString("enterTagName2") + " " + element.label,"");
	if(name) {
		gsCreateElement(element, "input", name, ["type", type]);
	}

	gsUpdateFormElementListBox();
}

function gsSetFormTargetToNewWindow() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	/* create params array */
	var param = ["target", "_new"];

	/* execute action */
	var action = new GsAction(GSACTION_ADD_ATTRIBUTE, element, param);
	gsDataModel.performAction(action);

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

	alert(getLocaleString("target1") + " " + element.getName() + " " + getLocaleString("target2"));
} 

function gsChangeToText() {

	function selectOption(aNode) {
		//alert(aNode.tagName.toUpperCase());
		switch(aNode.tagName.toUpperCase()) { // select the tags we are interested in, add more here...
			case "OPTION":
				//alert("found an option!");
				return NodeFilter.FILTER_ACCEPT;
		}
		return NodeFilter.FILTER_SKIP;
	}

	/* Grab document object from current selected tab */
	var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	var browserWindow = windowManager.getMostRecentWindow("navigator:browser").document.getElementById("content");
	var current_document = browserWindow.contentDocument

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	if(element.node.tagName == "INPUT") {

		/* create params array */
		var params = ["type", "text"];

		/* execute create action */
		var action = new GsAction(GSACTION_CLONE_ELEMENT, element, params);
		gsDataModel.performAction(action);
	}
	else if(element.node.tagName == "SELECT") {

		//alert('inside select transform');
		/* get a list of value */
		/*var options = [];*/
		var iterator = current_document.createTreeWalker(element.node, NodeFilter.SHOW_ELEMENT, selectOption, true);
		var input = iterator.nextNode();

		var table = current_document.createElement("table");
		table.setAttribute("style", "font-size: xx-small; border: 1px solid black; margin-top: 5px");

		var tr = current_document.createElement("tr");
		var td = current_document.createElement("td");

		tr = current_document.createElement("tr");
		td = current_document.createElement("td");

		td.appendChild(current_document.createTextNode(getLocaleString("selectCheatSheet") + " " + element.getName()));
		td.setAttribute("colspan", "2");
		td.setAttribute("style", "background: black; color: white; font-weight: bold");
		tr.appendChild(td);
		table.appendChild(tr);

		tr = current_document.createElement("tr");
		td = current_document.createElement("td");

		td.appendChild(current_document.createTextNode("Label"));
		td.setAttribute("style", "background: black; color: white");
		tr.appendChild(td);

		var td2 = current_document.createElement("td");
		td2.appendChild(current_document.createTextNode("Value"));
		td2.setAttribute("style", "background: black; color: white");
		tr.appendChild(td2);

		table.appendChild(tr);
		
		while (input) {
			tr = current_document.createElement("tr");
			td = current_document.createElement("td");
			td2 = current_document.createElement("td");

			td.appendChild(current_document.createTextNode(input.firstChild?(input.firstChild.data):("")));
			td2.appendChild(current_document.createTextNode(input.getAttribute("value")));
			td.setAttribute("style", "background: silver;");
			td2.setAttribute("style", "background: silver;");


			tr.appendChild(td);
			tr.appendChild(td2);

			table.appendChild(tr);

			/* alert("adding child : " +input.tagName); */
			/* alert(input.getAttribute("value") + " / " + input.firstChild.data); */
			/* options.push([input.getAttribute("value"), input.firstChild.data]); */
			input = iterator.nextNode();

		}

		var params = ["input", table, "name", element.getName(), "type", "text"];
		
		/* execute replace action */
		var action = new GsAction(GSACTION_REPLACE_ELEMENT, element, params);
		gsDataModel.performAction(action);
	
	}
	else {
		return;
	}


	//gsUpdateFormElementListBox();

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);


}

function gsRemoveFormElement() {


	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	/* create params array */
	var params = [];

	/* execute create action */
	var action = new GsAction(GSACTION_REMOVE_ELEMENT, element, params);
	gsDataModel.performAction(action);


	/* element.node.parentNode.removeChild(element.node); */

	gsUpdateFormElementListBox();
}


function gsRemoveAllEventHandlers() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	for(var i = 0; element.node.attributes.length > i; i++) {
		var attribute = element.node.attributes[i].name;

		if(attribute.toLowerCase().match(/(onabort|onblur|onchange|onclick|ondblclick|ondragdrop|onerror|onfocus|onkeydown|onkeypress|onkeyup|onload|onmousedown|onmouseup|onmousemove|onmouseout|onmouseover|onmouseup|onmove|onreset|onselect|onsubmit)/)) {

			/* create params array */
			var param = [attribute];

			/* execute action */
			var action = new GsAction(GSACTION_REMOVE_ATTRIBUTE, element, param);
			gsDataModel.performAction(action);
		}
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsRemoveLengthLimits() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);
	
	for(var i = 0; element.node.attributes.length > i; i++) {
		var attribute = element.node.attributes[i].name;

		if(attribute.toLowerCase().match(/(maxlength)/)) {

			/* create params array */
			var param = [attribute];

			/* execute action */
			var action = new GsAction(GSACTION_REMOVE_ATTRIBUTE, element, param);
			gsDataModel.performAction(action);
		}
	}
}

function gsModifyInputSize() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	/* prompt user for a new value */

	var newValue = prompt(getLocaleString("setSize"), element.node.getAttribute("size"));

	if(newValue) {
		/* create params array */
		var param = ["size", newValue];

		/* execute action */
		var action = new GsAction(GSACTION_CHANGE_ATTRIBUTE, element, param);
		gsDataModel.performAction(action);

		/* refresh attribute box */
		gsFillFormAttributes(gsFormItemsListBox, element.node);
	}

}

function gsAddTextNote() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	var text = prompt(getLocaleString("addNote") + " " + element.getName(), "");

	if(text) {
		element.addNoteAround(element.getName() + " (note)", text, "orange");
	}
}

function gsChangeFormMethod() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	if(element.getType() != "FORM") return;

	/* prompt user for a new value */
	var newValue = prompt(getLocaleString("newMethod") + " " + element.getName(), element.node.getAttribute("method"));

	if(newValue) {
		/* create params array */
		var param = ["method", newValue];

		/* execute action */
		var action = new GsAction(GSACTION_CHANGE_ATTRIBUTE, element, param);
		gsDataModel.performAction(action);

		/* refresh attribute box */
		gsFillFormAttributes(gsFormItemsListBox, element.node);
	}

}

function gsChangeAllHiddenToText() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");


	for(var i = 0; i < element.children.length; i++) {

		var field = element.children[i];
		if(!field.node.getAttribute("type")) continue;
		if(field.node.getAttribute("type").toUpperCase() != "HIDDEN") continue;

		/* create params array */
		var params = ["type", "text"];

		/* execute create action */
		var action = new GsAction(GSACTION_CLONE_ELEMENT, field, params);
		gsDataModel.performAction(action);

	}
	
	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}



/* Popup Option Menu things */

function gsSaveFormValues() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	if(element.node.tagName != "FORM") { alert(getLocaleString("chooseFormBefore")); return; }
	
	var newValue = prompt(getLocaleString("chooseNameToSave"));

	if(!newValue) return;


	gsDataModel.saveFormValues(newValue, selectedItem);

}

function gsLoadFormValues(val, index) {

	alert("Loading " + gsDataModel.savedFormValues[index][0]);

	for(var i = 0; i < gsDataModel.forms.length; i++) {
		var form = gsDataModel.forms[i];

		for(var j = 0; j < gsDataModel.savedFormValues[index][1].length; j++) {
			var formField = gsDataModel.savedFormValues[index][1][j];
			var fieldIndex = form.hasField(formField);
			//alert("testing " + formField.name + " on " + form.getName() + ". Result: " + fieldIndex);
			if( fieldIndex != -1) form.fillField(fieldIndex, formField);
		}
	}
}

function gsClearSavedFormSets() {

	if(confirm(getLocaleString("clearSavedForms"))) gsDataModel.savedFormValues = new Array();

}

function gsFillFormsItemListBox() {

	gsUpdateFormElementListBox();
}


/************************************************************************/
/* Functions used to perform encoding and decoding (and hashing)        */
/************************************************************************/

/* creates an instance of the encoder */
var encoder = new GsEncodingToolbox();

function gsBase64EncodeText() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.encodeBase64(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

function gsBase64DecodeText() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.decodeBase64(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

function gsHexEncodeText() {
	
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.encodeHex(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

function gsHexDecodeText() {
	
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.decodeHex(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

function gsHexMd5() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.hex_md5(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

function gsB64Md5() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.b64_md5(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsStrMd5() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.str_md5(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsHexSha1() {

	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.hex_sha1(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);

}

function gsB64Sha1() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.b64_sha1(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsStrSha1() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.str_sha1(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsHtlmNumEnt(val) {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.htmlNumEntEncode(val, value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsHtlmEnt() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.htmlEntEncode(value);
		//element.node.value = escape(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}


function gsURLEncode() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = escape(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}


function gsURLDecode() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = unescape(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsUnicodeEncode(val) {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.encodeUnicode(val, value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsHexEscapedEncode() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = encoder.encodeHexEscaped(value);
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsToUpperCase() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = value.toUpperCase();
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}

function gsToLowerCase() {
	/* get the element */
	var gsFormElementListBox = document.getElementById("gsFormElementListBox");
	var selectedItem = gsFormElementListBox.currentIndex;
	var element = gsDataModel.getNode(selectedItem);

	/* get the attribute box */
	var gsFormItemsListBox = document.getElementById("gsFormItemsListBox");

	var value = element.node.value;
	if(value) {
		element.node.value = value.toLowerCase();
	}

	/* refresh attribute box */
	gsFillFormAttributes(gsFormItemsListBox, element.node);
}
