/**
 *  WebsecurifyFirefoxService.js
 *  Copyright (C) 2007-2011  GNUCITIZEN
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

const CR = Components.results;
const CC = Components.classes;
const CI = Components.interfaces;

/* ------------------------------------------------------------------------ */

const CHROMEBASE = 'org.gnucitizen.websecurify.firefox';

/* ------------------------------------------------------------------------ */

Components.utils.import('resource://gre/modules/XPCOMUtils.jsm');

/* ------------------------------------------------------------------------ */

function WebsecurifyFirefoxService() {
	// pass
}

WebsecurifyFirefoxService.prototype = {
	classDescription: 'Websecurify Firefox Service',
	classID: Components.ID('{d341d111-6af2-4330-9202-44179aba7597}'),
	contractID: '@firefox.websecurify.gnucitizen.org/service;1',
	QueryInterface: XPCOMUtils.generateQI([CI.nsIObserver]),
	
	/* -------------------------------------------------------------------- */
	
	get wrappedJSObject () {
		return this;
	},
	
	/* -------------------------------------------------------------------- */
	
	observe: function (subject, topic, data) {
		if (topic == 'profile-after-change') {
			this.initializeComponent(subject, topic, data);
		} else
		if (topic == 'profile-before-change') {
			this.deinitializeComponent(subject, topic, data);
		} else
		if (topic == 'final-ui-startup') {
			this.initializeUi(subject, topic, data);
		}
		
		if (topic == 'http-on-examine-response' || topic == 'http-on-examine-cached-response') {
			this.observeResponse(subject, topic, data);
		}
	},
	
	/* -------------------------------------------------------------------- */
	
	initializeComponent: function (subject, topic, data) {
		let observerService = CC['@mozilla.org/observer-service;1'].getService(CI.nsIObserverService);
		
		observerService.addObserver(this, 'final-ui-startup', false);
		observerService.addObserver(this, 'http-on-examine-response', false);
		observerService.addObserver(this, 'http-on-examine-cached-response', false);
	},
	
	deinitializeComponent: function (subject, topic, data) {
		let observerService = CC['@mozilla.org/observer-service;1'].getService(CI.nsIObserverService);
		
		observerService.removeObserver(this, 'final-ui-startup');
		observerService.removeObserver(this, 'http-on-examine-response');
		observerService.removeObserver(this, 'http-on-examine-cached-response');
	},
	
	/* -------------------------------------------------------------------- */
	
	initializeUi: function () {
		let xulRuntime = CC['@mozilla.org/xre/app-info;1'].getService(CI.nsIXULRuntime);
		
		if (xulRuntime.OS != 'Darwin') {
			let windowWatcher = CC['@mozilla.org/embedcomp/window-watcher;1'].getService(CI.nsIWindowWatcher);
			
			windowWatcher.openWindow(null, 'chrome://' + CHROMEBASE + '/content/xul/backgroundPage.xul', 'websecurifyBackgroundPage', 'chrome,titlebar=no,popup=yes', null);
		}
	},
	
	/* -------------------------------------------------------------------- */
	
	observeResponse: function (subject, topic, data) {
		let httpChannel = null;
		
		try {
			httpChannel = subject.QueryInterface(CI.nsIHttpChannel);
		} catch (e) {
			return;
		}
		
		let xWebsecurifyRequest = 'false';
		
		try {
			xWebsecurifyRequest = httpChannel.getRequestHeader('X-Websecurify-Request');
		} catch (e) {
			// pass
		}
		
		if (xWebsecurifyRequest != 'true') {
			return;
		}
		
		try {
			let location = httpChannel.getResponseHeader('Location');
			
			if (location) {
				httpChannel.setResponseHeader('Location', null, false);
				httpChannel.setResponseHeader('X-Location-Override-Header', httpChannel.URI.spec, false);
			} else {
				httpChannel.setResponseHeader('X-Location-Override-Header', null, false);
			}
		} catch (e) {
			// pass
		}
		
		try {
			let wwwAuthentication = httpChannel.getResponseHeader('WWW-Authenticate');
			
			if (wwwAuthentication) {
				httpChannel.setResponseHeader('WWW-Authenticate', null, false);
				httpChannel.setResponseHeader('X-WWW-Authenticate-Override-Header', wwwAuthentication, false);
			} else {
				httpChannel.setResponseHeader('X-WWW-Authenticate-Override-Header', null, false);
			}
		} catch (e) {
			// pass
		}
	}
};

/* ------------------------------------------------------------------------ */

let NSGetFactory = XPCOMUtils.generateNSGetFactory([WebsecurifyFirefoxService]);

/*  GNUCITIZEN (Information Security Think Tank)
 **********************************************/