/*******************************************************************************
 * ***** BEGIN LICENSE BLOCK Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * 
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Mozilla.
 * 
 * The Initial Developer of the Original Code is IBM Corporation. Portions
 * created by IBM Corporation are Copyright (C) 2004 IBM Corporation. All Rights
 * Reserved.
 * 
 * Contributor(s): Darin Fisher <darin@meer.net> Doron Rosenberg
 * <doronr@us.ibm.com>
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or the
 * GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
 * case the provisions of the GPL or the LGPL are applicable instead of those
 * above. If you wish to allow use of your version of this file only under the
 * terms of either the GPL or the LGPL, and not to allow others to use your
 * version of this file under the terms of the MPL, indicate your decision by
 * deleting the provisions above and replace them with the notice and other
 * provisions required by the GPL or the LGPL. If you do not delete the
 * provisions above, a recipient may use your version of this file under the
 * terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK *****
 */

// edited by ebricca 2004/04/16
// came from the magpie / bkstr implementation from bengoodger to this
// ideas also from the livehttpheaders -> observer



// httpmod protocol related
// is started via httpmod:start
// is stopped via httpmod:stop
// also checks on every query a pref if its still active -- may tamper speed

const kSCHEME = "httpmod";
const kPROTOCOL_NAME = "httpmod Protocol";
const kPROTOCOL_CONTRACTID = "@mozilla.org/network/protocol;1?name=" + kSCHEME;
const kPROTOCOL_CID = Components.ID("C2E923E4-DBA4-410A-B225-D472DA09A0E2");

// Mozilla defined
const kSIMPLEURI_CONTRACTID = "@mozilla.org/network/simple-uri;1";
const kIOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";
const nsISupports = Components.interfaces.nsISupports;
const nsIIOService = Components.interfaces.nsIIOService;
const nsIProtocolHandler = Components.interfaces.nsIProtocolHandler;
const nsIURI = Components.interfaces.nsIURI;



// logging
var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
function dumpme(aString)
{
 aConsoleService.logStringMessage(aString);
}

function Protocol()
{
}

Protocol.prototype =
{
  //

  QueryInterface: function(iid)
  {
    if (!iid.equals(nsIProtocolHandler) &&
        !iid.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;


    return this;
  },

  scheme: kSCHEME,
  defaultPort: -1,
  protocolFlags: nsIProtocolHandler.URI_NORELATIVE |
                 nsIProtocolHandler.URI_NOAUTH,
  
  allowPort: function(port, scheme)
  {
    return false;
  },

  newURI: function(spec, charset, baseURI)
  {
    var uri = Components.classes[kSIMPLEURI_CONTRACTID].createInstance(nsIURI);
    uri.spec = spec;   
    return uri;
  },

  newChannel: function(aURI)
  {
      var ios = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
      // doesnt need .. hack against crash by providing a correct return
        // channel
      var dumbnsiURI  = ios.newURI("about:blank", null, null);
      var dumbChannel = ios.newChannelFromURI(dumbnsiURI).QueryInterface(Components.interfaces.nsIChannel);


      
      // mind that this default deselects it -- if pref wasnt defined yet
      xpPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
      if (xpPref.getPrefType("refspoof.customRefState") == 0) {
          xpPref.setIntPref("refspoof.customRefState", 0);
      }
      // sets default against exeption 
      if (xpPref.getPrefType("refspoof.refCustomUri") == 0) {
          xpPref.setCharPref("refspoof.refCustomUri", "http://google.com");
      }
      
    var myURI = aURI.spec;
    
    // dumpme(myURI);


    if (myURI == "httpmod:start") {
      // on start we activate our "http-on-modify-request"
      this.observe(null,"httpmod-activate",null);
    } else if (myURI == "httpmod:stop") {
        // on stop we kill our "http-on-modify-request" observer
        this.killModify();
    }
    
    //add httpmod-activate as observer opt for no back forth comm
    //var obsService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    //obsService.addObserver(this, "httpmod-activate", false);
    
    return dumbChannel;
  },

  killModify : function ()
  {
      var obsService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obsService.removeObserver(this, "http-on-modify-request", false);

  },
  
  addObserver : function (observer)
  {
    this.observers[observer] = observer;
  },

  removeObserver : function (observer)
  {
    delete this.observers[observer];
  },

  observe: function(aSubject, aTopic, aData) {
    if (aTopic == 'http-on-modify-request') {
        aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
        this.onModifyRequest(aSubject);
        
    } else if (aTopic == 'httpmod-activate') {
        if ('nsINetModuleMgr' in Components.interfaces) {
            dumpme("use a new moz version after september 15, 2003 \n");
            return;
        } else {
            var obsService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
 
            // i opt for the easy solution means multiple start invocation .. no broadcasts
            //xpPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
            //xpPref.setIntPref("refspoof.httpModIsActive", 1);            
            // suppose a kill isnt nesessary -- new array should initialize again
            
            obsService.addObserver(this, "http-on-modify-request", false);
            // Initialisation of all needed vars .. dont know how it works in detail
            // yet
            this.observers = new Array();
            this.delays    = new Array();
        }

    } else {
        dumpme("nsHeaderInfo: unknown TOPIC: " + aTopic + "\n");
    }
  },

  onModifyRequest : function (oHttp)
  {
      // check everytime before overriding referrer with custom/static that we have still rights
      xpPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
      refStateCtrl = xpPref.getIntPref("refspoof.customRefState");

      
      if (refStateCtrl==0){
          this.killModify();
          return;
      }
      
      
      // id like to get access to browser window thoug dont see how yet
      oHttp.QueryInterface(Components.interfaces.nsIHttpChannel);
      ios = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
      
      refCustomUri = xpPref.getCharPref("refspoof.refCustomUri");
      refnsiURI  = ios.newURI(refCustomUri, null, null);

      oHttp.referrer=refnsiURI;
      
      dumpme(oHttp.URI.spec);
      dumpme(oHttp.originalURI.spec);
      dumpme(oHttp.owner);
      dumpme("----");

  },

}

var ProtocolFactory = new Object();

ProtocolFactory.createInstance = function (outer, iid)
{
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;

  if (!iid.equals(nsIProtocolHandler) &&
      !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  return new Protocol();
}


/**
 * JS XPCOM component registration goop:
 * 
 * We set ourselves up to observe the xpcom-startup category. This provides us
 * with a starting point.
 */

var nsHttpModProtocolModule = new Object();

nsHttpModProtocolModule.registerSelf = function (compMgr, fileSpec, location, type)
{
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(kPROTOCOL_CID,
                                  kPROTOCOL_NAME,
                                  kPROTOCOL_CONTRACTID,
                                  fileSpec, 
                                  location, 
                                  type);
}

nsHttpModProtocolModule.getClassObject = function (compMgr, cid, iid)
{
  if (!cid.equals(kPROTOCOL_CID))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  if (!iid.equals(Components.interfaces.nsIFactory))
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
  return ProtocolFactory;
}

nsHttpModProtocolModule.canUnload = function (compMgr)
{
  return true;
}

function NSGetModule(compMgr, fileSpec)
{
  return nsHttpModProtocolModule;
}
